% function [val, theta] = BRDF( Ein, xIn, L, thetaMax, dTheta)
% BRDF of Ein, theta = radius / L, thetaMax is largest angle to calculate, dTheta is bin size of angle
% E.g., [E,x] = fp.getField('ETM-HR-i'), [val, theta] = BRDF(E,x,4000,5e-4)
%
% May 24, 2020
%   First release in GIT
%   This is independent of SIS
%
function [val, theta] = BRDF( Ein, xIn, L, thetaMax, dTheta)
if ~exist( 'dTheta', 'var' )
    N = 100;
    dTheta = thetaMax / N;
else
    N = floor( thetaMax/dTheta + 0.001 );
end

rOut = dTheta * (1:N) * L;
% cannot interpolate out of range
rOut( rOut > xIn(end) ) = [];
% Eout(x,y) to Pout(r)
Pout = UtilTK.xy2r( xIn, abs(Ein).^2, rOut );
theta = rOut / L;

Ptot = sum(sum(abs(Ein).^2))*(xIn(2)-xIn(1))^2;
val = Pout * L^2 / Ptot;
