function [freq, ASD_2D] = LMA_ASD( map, x, y, diam, doShift )
% function [freq, ASD_2D] = LMA_ASD( surface, xl, yl, diam )
% calculate 2D ASD
%     surface : height of the surface in m
%     xl, yl  : grid points of data
%     diam    : mirror diameter
%
% originaly PLOT_PSD by Jerome Degallaix <j.degallaix@lma.in2p3.fr>
% Adapted from a set of functions made by M. Galimberti in the years
% 2010-2011 at LMA.

% -------------------------------------------------------------------------------
%            First prepare the map for the calculation

if min( size(x) ) == 1 && min( size(y) ) == 1
    [x, y] = meshgrid( x, y );
end
rsq = x.^2 + y.^2;
    
% Remove the NaN if any
map(isnan(map)) = 0;

% Remove the average over the central part
map = map - mean(map(rsq <= (diam/2)^2));

% fill 0 outside
map(rsq > (diam/2)^2) = 0;

% Cut the map around the useful part (a square of length diam_PSD)
% make the area square
indx = find(abs(x(1,:)) <= diam/2);
if mod( length(indx), 2 ) ~= 0
    indx = indx(1:end-1);
end
indy = find(abs(y(:,1)) <= diam/2);
if mod( length(indy), 2 ) ~= 0
    indy = indy(1:end-1);
end
dleng = (length( indx ) - length( indy ))/2;
if dleng > 0
    indx = indx( 1+dleng:(end-dleng) );
elseif dleng < 0
    indy = indy( 1-dleng:(end+dleng) );
end

map2 = map(indy,indx);
N_point = length(indx);

w = hanning(N_point);
m = w(:)*w(:).'; % Outer product window

P_window =  (sum(sum(abs(m).^2))/N_point^2); % Power of the window

map2 = map2.*m/sqrt(P_window);

%------------------------------------------------------------------------
% Calculate the PSD 2D then put it in 1D

% Now can do the PSD 2D
ASD_2D = fft2(ifftshift(map2)) * (x(1,2)-x(1,1))^2;  % !! low frequency at the corner now
freq = (0: N_point/2) * 1/diam;

if exist( 'doShift', 'var' ) && doShift
    ASD_2D = fftshift(ASD_2D);
    freq = (-N_point/2:N_point/2-1) / diam;
end

