%
% function [fname, RoC] = TMInfo( TMkind, TMNO, dirName )
%   TMkind = 'ITM' or 'ETM'
%   TMNO is the test mass number, like ETM07
%   dirName is the folder relatie to the caller. 
%   If none, '/Users/hiro/Dropbox/RichardDay/FOGPrime13/Data/' is used. (March 2nd, 2017)
% 
% September 23, 2019
%   data for O3 optics
%   TMNO = -1 ~ -4 for LHO X, LHO Y, LLO X, LLO Y
%
% April 8, 2019
%   more data and clean up
%
% Jan. 30, 2019
%   more data and transmittance added in data
%
function [fname, RoC, T, HRL] = TMInfo( TMkind, TMNO, dirName )

ITMIDs = [7, 11, 4, 8];
ETMIDs = [13, 16, 10, 15];
if TMNO < 0
    if TMkind == 'I'
        TMNO = ITMIDs(-TMNO);
    else
        TMNO = ETMIDs(-TMNO);
    end
end

% LHO
ITM07dat = 'ITM07_-power_in_160mm.dat';  
T_ITM07 = 1.50e-2; RoC_ITM07 = 1940.3;
ITM11dat = 'ITM11_CIT.dat';              
T_ITM11 = 1.42e-2; RoC_ITM11 = 1939.2;

ETM13dat = 'ETM13_-power_160.dat';
RoC_ETM13 = 2244.2; T_ETM13 = 3.9e-6; % https://dcc.ligo.org/LIGO-E1500376
ETM16dat = 'ETM16_S1_-power_on_160.dat';
RoC_ETM16 = 2246.9; T_ETM16 = 3.8e-6; % https://dcc.ligo.org/LIGO-E1500254

%LLO
ITM04dat = 'ITM04_CITZygo.dat';  
T_ITM04 = 1.48e-2; RoC_ITM04 = 1937.9;
ITM08dat = 'ITM08_CITZygo.dat';  
T_ITM08 = 1.48e-2; RoC_ITM08 = 1940.7;

ETM10dat = 'ETM10_S1-power.dat';
RoC_ETM10 = 2247.7; T_ETM10 = 4e-6; HRL_ETM10 = 7.2e-6; % https://dcc.ligo.org/LIGO-E1500377  
ETM15dat = 'ETM15_S1_-power_fit_on_160mm.dat';
RoC_ETM15 = 2245.4; T_ETM15 = 3.9e-6; HRL_ETM15 = 9.1e-6; % https://dcc.ligo.org/LIGO-E1500375


CP03dat = 'CP03_Trans_viewed-from-S1.dat';
CP06dat = 'CP06_sptwe.dat';
CP05dat = 'xCP_05_SPTWE_Figure.dat';
CP01dat = 'xCP_01_SPTWE_Figure.dat';
RoC_CP03 = 1e20;
RoC_CP06 = 1e20;
RoC_CP05 = 1e20;
RoC_CP01 = 1e20;

ITM03dat = 'ITM03_CIT.dat';      T_ITM03 = 1.39e-2;

SPTITM04dat = 'itm04_sptwe.dat';
SPTITM08dat = 'itm08_sptwe.dat';
SPTITM03dat = 'xITM_03_SPTWE_Figure.dat';
SPTITM11dat = 'xITM_11_SPTWE_Figure.dat';

ITM06dat = 'ITM06_S1_-Power_fit_160.dat';
ITM10dat = 'ITM10_S1_power_on_160_sub.dat';
ITM05dat = 'ITM05_S1_1X_-P.dat';
ITM01dat = 'ITM01_S1-P_160.dat';

RoC_SPTITM04 = 302.41e3;
RoC_SPTITM08 = -82e3;
RoC_ITM03 = 1939.3;
RoC_SPTITM03 = -79.70e3;
RoC_SPTITM11 = 570.0e3;

RoC_ITM06 = 1934.34;

RoC_ITM10 = 1939.2;
RoC_ITM05 = 1939.7;
RoC_ITM01 = 1940.2;

ETM03dat = 'ETM03.dat';
ETM06dat = 'ETM06.dat';
ETM07dat = 'ETM07_CIT.dat';
ETM09dat = 'ETM09_CIT.dat';
ETM08dat = 'ETM08_CIT.dat';
ETM12dat = 'ETM12_CIT.dat';
ETM11dat = 'ETM11_CIT.dat';
ETM14dat = 'ETM14_CIT.dat';

RoC_ETM03 = 2309.7; T_ETM03 = 4.8e-6;
RoC_ETM06 = 2196.9; T_ETM06 = 4.6e-6;

RoC_ETM07 = 2239.7; % https://dcc.ligo.org/LIGO-E1300365
RoC_ETM09 = 2242.4; T_ETM09 = 3.6e-6; % https://dcc.ligo.org/LIGO-E1300431
RoC_ETM08 = 2241.54; % https://dcc.ligo.org/LIGO-E1300365
RoC_ETM12 = 2238.9; % https://dcc.ligo.org/LIGO-E1300431

RoC_ETM11 = 2250.0;  T_ETM11 = 4.5e-6; HRL_ETM11 = 21e-6; % https://dcc.ligo.org/LIGO-E1400448
RoC_ETM14 = 2250.99; T_ETM14 = 4.5e-6; HRL_ETM14 = 18e-6;

HRL = 10e-6;
if TMkind(1) == 'I'
    TMkind = 'ITM';
    RoC = 1934;
    T = 0.014;
elseif TMkind(1) == 'C'
    TMkind = 'CP';
    RoC = 1e20;
    T = 1;
elseif TMkind(1) == 'S' % SPT of ITM
    TKkind = 'SPTITM';
    RoC = 1e20;
    T = 1;
else
    TMkind = 'ETM';
    RoC = 2245;
    T = 5e-6;
end

if TMNO == 0
    fname = '';
    return;
end

fnameVar = strcat(TMkind, num2str(TMNO, '%2.2d'), 'dat');
if ~exist( fnameVar, 'var' )
    fname = '';
else
    if ~exist('dirName', 'var')
        dirName = '/Users/hiro/Dropbox/RichardDay/FOGPrime13/Data/';
    end

    eval(strcat('fname = ', fnameVar, ';'));
    fname = strcat( dirName, fname );
    
    varList = {'RoC', 'T', 'HRL'};
    for ivar = 1 : length( varList )
        varName = varList{ivar};
        str = strcat( varName, '_', TMkind, num2str(TMNO, '%2.2d') );
        if exist( str, 'var' )
            eval( strcat(varName, '=', str, ';' ) );
         end
    end
end
