% classdef UtilTK
%   general purpose static functions
%
% May 13, 2020
%  function dmap = difAmp( xlist, ampList, rx0, ry0, x0, y0, rx1, ry1, x1, y1 ) added
%     calculate fields with displaced baffles
%
% April 26, 2020
%  dndr = xy2r( xlist, xyDist, rList, x0, y0 ) improved
%
% April 7, 2020
%  function dndr = xy2r( xlist, xyDist, rList, x0, y0 ) added to calculate integral over a ring
%
% December 21, 2019
%  function Ival = integrate( val, dx, asIs [false] ) added
%           Ival = integrate over the first or second whichever is larger
%
% November 19, 2019
%   setVARARGS accepts array of numerical values, like (10:10:50)
%
% October 7, 2019
%   sumIn supports offset (x0,y0), which is the center of the circle
%
% Sep. 26, 2019
%   sumOut removed, because sumOut(r) = sumIn(-r)
%
% Sep. 24, 2019
%   sumIn and sumOut uses SqInCircle for better boundary handling
%
% June 25, 2019
% [x, val] = findMax( xList, valList ) added
%    find peak of valList. Values around the peak is used to find the maximal point by polyfit
%
% June 5th, 2019
%  function str = setVars( nameList, showIt, varargin )
%   Modified so that '?' prints out available name and avlues.
%   if varargin{1} starts with '?', nameList is printed whose name match with '.*' + varargin{1}(2:end) + '.*'
%
% May 3rd, 2019
%    function val = getVar( varName, showIt, varargin ) added
%    val = getVar( varName, showIt, varargin ) can be used to check if varName is in the inputs,
%    instead of calling setVars.
%
% May 2nd, 2019
%    function ind = wildMatch( strList, pat ) added to find items in strList matching with the pat with wildcard
%    strList = {ifo.fields.fieldList(:).name}, pat = BS*x*
%
% July 30, 2018
%    setVars now accepts numerical array value, like [1,2,3;4,5,6+2i]
%
% April 25, 2017
%    new function argList = pickArg( str ) added
%    For str = '(a,b,c)', this returns a,b,c in argList
%
% March 8, 2017
%    new function
%    setVARARGS( varargin, {'name1', defVal1, 'name2', defVal2, ..., 'nameN', defValN)} )
%    added for the default input specification, like lock( 1e-5, 10 )
%
% March 6, 2017
%    useQuarter( mat4, Nsmall ) has new argument, Nsmall.
%    This picks up central Nsmall x Nsmall rectangle as output. Default is half size of the input mat4
%
% August 11, 2016
%    stdaveW, weighted std and average
%
% February 4, 2016
%    setDEFVAL
%
% October 23, 2015
%    r2 = XYsq( x, y, x0, y0, rx, ry ) added
%
% September 23, 2015
%    In lowPassMap, (i)fftshift are called appropriately. Fortunately, old
%    codes worked OK.
%
% September 16, 2015
%    function valout = interp2grid( valin, xin, yin, xout, yout )
%       for simple 2D interpolation
%
% September 11, 2015
%   pickCircle : extract data in a circle around a specified center
%
% July 3d, 2015
%   two functions, evalError and areSameVecs, are moved from cavityManager.
%
% August 12, 2014
%   mapped = mapping( x, y, mat, newX, newY ) added, which rotates mat(iy,ix) using the formula given in newX and newY
%
classdef UtilTK
    
    methods (Static)
        % eval( UtilTK.setDEFVALS('name1', val1, 'name2', val2, ...) )
        %
        % example : ifo = LHOIFO('Pabs', 1e-3)
        %
        % for each set of 'nameN', and valN, the following is created
        %    if (~exist('varName', 'var') || length(val) == 0) varName = val; end;
        function cmd = setDEFVALS( varargin )
            cmd = '';
            for n = 1: 2: length( varargin )
                cmd = strcat( cmd, 'if (~exist(''', varargin{n}, ''', ''var'') || length(', varargin{n}, ') == 0) ', varargin{n}, ' = ' );
                if ischar( varargin{n+1} )
                    cmd = strcat( cmd, '''', varargin{n+1}, '''');
                else
                    cmd = sprintf('%s %18.16e', cmd, varargin{n+1});
                end
                cmd = strcat( cmd, '; end;' );
            end
        end
        
        % eval( setVARARGS( varargin, {'name1', defVal1, 'name2', defVal2, ..., 'nameN', defValN)} )
        %
        % example : lock( 1e-4 )
        %
        % when the function is called with some arguments, first value is used for variable name1, second for name
        % and, for those variables nameN, nameN+1, etc, which are not passed values for, the default values are used.
        % E.g., lock(1e-4) is called, and sevVARARGS( varargin, {'error', 1e-7, 'Nprint', 10} ) is called,
        % error is set to be 1e-4, and Nprint is set to default value of 10.
        % This is equivalent to
        %    for id <= length( varargin )
        %       name(id) = varargin{id}
        %    for id > length( varargin )
        %       name(id) = defVal{id}
        function cmd = setVARARGS( varargin, specs )
            cmd = '';
            for id = 1 : length(specs)/2
                cmd = strcat( cmd, specs{id*2-1}, '=' );
                if id <= length( varargin )
                    val = varargin{id};
                else
                    val = specs{id*2};
                end
                if ischar( val )
                    cmd = strcat( cmd, '''', val, ''';');
                else
%                    cmd = sprintf('%s %18.16e;', cmd, val);
                    rhs = '[ ';
                    for ir = 1 : size( val,1 )
                        for ic = 1 : size( val,2 )
                            rhs = sprintf('%s %s ', rhs, num2str( val(ir,ic) ));
                        end
                        rhs = sprintf('%s ;', rhs);
                    end
                    % remove last ;
                    rhs = sprintf('%s ];', rhs(1:length(rhs)-1));
                    cmd = sprintf('%s %s', cmd, rhs);
                end
            end
        end
        
        % error = sqrt( abs(Eold-Enew)^2 / abs(Enew)^2 )
        function err = evalError( Eold, Enew )
            Ediff = Enew - Eold;
            err = sqrt( sum(sum( conj(Ediff) .* Ediff )) / sum(sum( conj(Enew).*Enew )) );
        end
        
        function val = areSameVecs( v1, v2 )
            val = length(v1) == length(v2);
            if val
                val = all( v1 == v2 );
            end
        end
        
        function mapped = mapping( x, y, dat, newX, newY )
            % mapped = mapping( x, y, dat, newX, newY )
            %   calculate data at new coodrinate system or changing the data by rotating, moving or expanding,
            %   x and y are 2D coordinates in the original frame, dat(Ny,Nx) are 2D data
            %   and newX and newY are values in the new coordinate system
            %   or mapped( x, y ) = mat( newy, newx ), or newX/newY defines coordinate change, opposite to object change
            %   example :
            %       1) newX = cos * x - sin * y, newY = sin * x + cos * y
            %          This will rotate mat
            %       2) newX = 2*x, newY = 0.5*y
            %          This will shrink object by factor 2 in x direction and expand by factor 2 in y direction
            if min( size(x) ) == 1 && min( size(y) ) == 1
                [x, y] = meshgrid( x, y );
            end
            
            dx = x(1,2) - x(1,1);   xbase = x(1,1) - dx/2;  Nx = size(x,2);
            dy = y(2,1) - y(1,1);   ybase = y(1,1) - dy/2;  Ny = size(y,1);
            
            % for x(ix),y(iy), the value at newx(x(ix),y(iy)), newy(x(ix),y(iy)), is used
            if ischar( newX )
                newX = eval( newX );
            end
            newX = floor( (newX - xbase)/dx ) + 1;
            newX = min( max( newX, 1 ), Nx );
            if ischar( newY )
                newY = eval( newY );
            end
            newY = floor( (newY - ybase)/dy ) + 1;
            newY = min( max( newY, 1 ), Ny );
            
            mapped = interp2( 1:Nx, 1:Ny, dat, newX, newY );
        end
        
        function valout = interp2grid( valin, xin, yin, xout, yout )
            % valin(1:Ny,1:Nx) at grid points xin(1:Nx) and yin(1:Ny) is used
            % to interpolate to get values at xout(1:Mx) and yout(1:My) in valout(1:My,1:Mx)
            % using a simple 1D fit, val = a x + b y + c
            dx = xin(2) - xin(1);
            dy = yin(2) - yin(1);
            nx0 = min( max( ceil((xout-xin(1))/dx + 0.5), 1 ), length(xin)-1 );
            ny0 = min( max( ceil((yout-yin(1))/dy + 0.5), 1 ), length(yin)-1 );
            val00 = valin(ny0, nx0);
            val10 = valin(ny0, nx0+1);
            val01 = valin(ny0+1, nx0);
            val11 = valin(ny0+1, nx0+1);
            C0 = (3*val00 + val10 + val01 - val11) / 4;
            Cx = (val11 + val10 - val01 - val00) / 2;
            Cy = (val11 - val10 + val01 - val00) / 2;
            [xout, yout] = meshgrid( (xout-xin(nx0))/dx, (yout-yin(ny0))/dy );
            valout = Cx .* xout + Cy .* yout + C0;
        end
        
        function valout = interp2serial( valin, xin, yin, xout, yout )
            % valin(1:Ny,1:Nx) at grid points xin(1:Nx) and yin(1:Ny) is used
            % to interpolate to get values at xout(1:Mx) and yout(1:My) in valout(1:My,1:Mx)
            % using a simple 1D fit, val = a x + b y + c
            dx = xin(2) - xin(1);
            dy = yin(2) - yin(1);
            nx0 = min( max( ceil((xout-xin(1))/dx + 0.5), 1 ), length(xin)-1 );
            ny0 = min( max( ceil((yout-yin(1))/dy + 0.5), 1 ), length(yin)-1 );
            rawLeng = size(valin,2);
            val00 = valin((ny0-1)*rawLeng + nx0);
            val10 = valin((ny0-1)*rawLeng + nx0+1);
            val01 = valin(ny0*rawLeng + nx0);
            val11 = valin(ny0*rawLeng + nx0+1);
            C0 = (3*val00 + val10 + val01 - val11) / 4;
            Cx = (val11 + val10 - val01 - val00) / 2;
            Cy = (val11 - val10 + val01 - val00) / 2;
            valout = Cx .* xout + Cy .* yout + C0;
        end
        
        function val = inputD( prompt, default )
            % val = inputD( prompt, default )
            % prompt and read one value, possibly with default
            val = input( prompt );
            if isempty( val ) && exist('default', 'var')
                val = default;
            end
        end
        
        function val = getVar( varName, showIt, varargin )
            % val = getVar( varName, showIt, varargin )
            %    this looks varName in varargin, just the same as setVars, and returns the value if exists
            %    if the varName is not in varargin, val is nan
            val = nan;
            ind = find( strcmp(varargin, varName ) );
            if ~isempty( ind )
                indind = find( mod(ind,2) == 1, 1 );
                if ~isempty( indind )
                    val = varargin{ ind(indind) + 1 };
                end
            end
            if showIt && ~isnan( val(1) )
                fprintf(1, '%s is defined to be ', varName); disp( val );
            end
        end
        
        function str = setVars( nameList, showIt, varargin )
            % str = setVars( prompt, nameList, varargin )
            %   find varargin{n} which is in a cell array, and create a string varargin{n} = varargin{n+1}
            %   if varargin{1} starts with '?', nameList is printed whose name match with '.*' + varargin{1}(2:end) + '.*'
            if length(varargin) >= 1 && varargin{1}(1) == '?'
                if length( varargin{1} ) > 1
                    pattern = strcat('.*', varargin{1}(2:end), '.*');
                else
                    pattern = '';
                end
                str = 'disp(''====''), obj,';
                for in = 1 : length(nameList)
                    % 'obj' and 'varargin' are not to be listed
                    if strcmp( nameList{in}, 'obj' ) == 0 && strcmp( nameList{in}, 'varargin' ) == 0
                        if ~isempty( pattern )
                            if isempty( regexpi( nameList{in}, pattern ) )
                                continue;
                            end
                        end
                        str = strcat( str, nameList{in}, ',' );
                    end
                end
                return;
            end
            in = 1;
            str = '';
            while in < length(varargin)
                if ischar( varargin{in} )
                    id = find( strcmp( varargin{in}, nameList ) == 1, 1 );
                    if ~isempty( id )
                        if ischar( varargin{in+1} )
                            rhs = sprintf('''%s''', varargin{in+1});
                        else
                            rhs = '[ ';
                            for ir = 1 : size( varargin{in+1},1 )
                                for ic = 1 : size( varargin{in+1},2 )
                                    rhs = sprintf('%s %s ', rhs, num2str( varargin{in+1}(ir,ic) ));
                                end
                                rhs = sprintf('%s ;', rhs);
                            end
                            % remove last ;
                            rhs = sprintf('%s ]', rhs(1:length(rhs)-1));
                        end
                        oneCmd = sprintf( '%s = %s', nameList{id}, rhs);
                        if showIt
                            disp( oneCmd );
                        end
                        str = sprintf('%s %s;', str, oneCmd );
                        in = in + 1;
                    end
                end
                in = in + 1;
            end
        end
        
        function str = setVarsInputs( prompt, varargin )
            % usage : eval( UtilTK.setVarsInputs( 'set values', 'leng', 'RoC' ) ) or
            %         eval( UtilTK.setVarsInputs( 'leng' ) )
            % User is promprted to enter command, like leng = 4000; RoC = 1934
            % and this function returns a string which can be executed using eval
            str = '';
            if nargin == 0
                return;
            end
            if nargin > 2 % more than two variables
                if isempty( prompt )
                    prompt = 'Set values';
                end
                fprintf(1, '%s (name = val, ? for list, return to end)', prompt );
                promptMark = ' > ';
            else
                if nargin == 1
                    varargin{1} = prompt;
                    prompt = '';
                end
                fprintf(1, '%s %s = ', prompt, varargin{1} );
                promptMark = '';
            end
            % reapeat until empty input
            while true
                command = input( promptMark, 's');
                if isempty( command )
                    break;
                end
                % analyze one command, separated by ;, at a time, until all are analyzed
                while ~isempty(command)
                    % oneCmd + ';' + new command = old command
                    [oneCmd, command] = UtilTK.extactOne( command );
                    if isempty( oneCmd )
                        continue;
                    end
                    if oneCmd == '?'
                        for ivar = 1 : length(varargin)
                            fprintf(1, '''%s''  ', varargin{ivar});
                        end
                        fprintf(1, '\n');
                    else
                        ind = find( oneCmd == '=', 1 );
                        % if assignment, and if name list is given, make sure the name is in the list
                        if isempty(ind)
                            if length( varargin ) == 1
                                oneCmd = sprintf( '%s = %s', varargin{1}, oneCmd );
                            else
                                fprintf(1, 'command should be name = val\n');
                                continue;
                            end
                        else
                            name = strtrim( oneCmd(1:ind-1) );
                            if isempty( find( strcmp(varargin,name), 1 ) )
                                fprintf(1, 'unknown name : %s\n', name );
                                continue;
                            end
                        end
                        str = sprintf( '%s %s;', str, oneCmd );
                    end
                end
                if nargin <= 2
                    break;
                end
            end
        end
        
        function updated = updateArgList( origList, newDef, opt )
            % function updated = updateArgList( origList, newDef, opt )
            % newDef ( 'name1', var1, 'name2', var2 ) is used to update the original list.
            % opt = 0 : replace value if name exists, add new name and var if not
            % opt = 1 : only replace, no merge, and continue
            % opt = 2 : only replace, print warning
            updated = origList;
            if isempty( newDef )
                return;
            end
            firstWarning = true;
            for idnew = 1 : 2 : length( newDef )
                id = UtilTK.tagInVararg( newDef{idnew}, updated );
                if id == 0
                    if opt == 0
                        curLeng = length( updated );
                        updated{ curLeng : curLeng+1 } = newDef{idNew:idNew+1};
                    elseif opt == 2
                        fprintf(1,'>>> variable ''%s'' is not in the name list\n', newDef{idnew} );
                        if firstWarning
                            firstWarning = false;
                            fprintf(1,'    names are :');
                            for id = 1 : 2 : length(updated)
                                fprintf(1,'''%s''\t', updated{id});
                            end
                            fprintf(1,'\n');
                        end
                    end
                else
                    updated{id+1} = newDef{idnew+1};
                end
            end
        end
        
        function [oneCmd, command] = extactOne( command )
            % [oneCmd, command] = extactOne( command )
            % extract one command separated by ; and return in oneCmd
            % command is the original with the oneCmd trimmed out
            separator = find( command == ';', 1 );
            if isempty( separator )
                oneCmd = command;
                command = '';
            else
                oneCmd = command(1:separator-1);
                command = command(separator+1:end);
            end
            oneCmd = strtrim( oneCmd );
            command = strtrim( command );
        end
        
        function id = namesInList( name, tagList )
            % find tag(s) in tagList which match with name.
            %    name = 'T', tagList = {'T', 'Thick'}
            %
            % If multiple tags match, like name = 'T' and tag ={'T', Thick'}
            % full match, 'T', is chosen.
            % If tagList = {'Thick', 'Tilt'}, both are returned.
            
            id = find( strncmp( name, tagList, length(name) ) );
            if length( id ) > 1
                for idu = id
                    if length( tagList{idu} ) == length(name)
                        id = idu;
                        break;
                    end
                end
            end
        end
        
        function id = oneNameInList( name, tagList )
            % find the unique tag ID using namesInList
            %    name = 'T', tagList = {'T', 'Thick'}
            %
            % If there is no match, return id = 0.
            % If there are multiple mathces, an error message is printed and exits
            id = UtilTK.namesInList( name, tagList );
            if isempty(id)
                id = 0;
            elseif length( id ) > 1
                str = sprintf('''%s''', tagList{id(1)});
                for idu = id(2:end)
                    if idu == id(end)
                        str = sprintf('%s and ''%s''', str, tagList{idu} );
                    else
                        str = sprintf('%s, ''%s''', str, tagList{idu} );
                    end
                end
                error( 'name ''%s'' matches with following candidates {%s}', name, str );
            end
        end
        
        function newList = identifyTags( tagList, varargin )
            % vararg is valid argments to specify parameters, pass validArgList.
            % tag names in vararg is replaced by tags in tagList.
            % names in vararg are replaced by the full names in tagList.
            % If there are any names which are not in tagList, return empty newList
            
            if ~UtilTK.validArgList( varargin{:} )
                error('argument needs to be a list of ''name'' and value pairs');
            end
            newList = varargin;
            for id = 1 : 2: length( newList )
                tagID = UtilTK.oneNameInList( newList{id}, tagList );
                if tagID == 0
                    newList = {};
                    return;
                end
                newList{id} = tagList{tagID};
            end
        end
        
        function [val, inputs] = extractVal( tag, inputs, defaultVal )
            % [val, inputs] = extractVal( tag, inputs, defaultVal )
            % for the input string, 'name1', val1, 'name2', val2, ...
            % find a 'nameX' which is in the tag list and returns the
            % associtaed value, valN.
            % When a nameX is found, the name section of the return value "inputs" is emptied
            id = UtilTK.tagInVararg( tag, inputs );
            if id == 0
                if exist( 'defaultVal', 'var' )
                    val = defaultVal;
                else
                    val = [];
                end
            else
                val = inputs{id+1};
                if nargout == 2
                    inputs{id} = '';
                end
            end
        end
        
        function updateProperties( object, tagList, inputs )
            % object.(tagName) = tagged value in vararg
            
            for tagID = 1 : length( tagList )
                id = UtilTK.tagInVararg( tagList(tagID), inputs );
                if id > 0
                    object.(tagList{tagID}) = inputs{ id + 1 };
                end
            end
        end
        
        
        function id = tagInVararg( tag, inputs )
            % id = tagInVararg( tag, inputs )
            % find tag names in vararg name section ( 'name1', val1, 'name2', val2, etc )
            
            id = find( strcmp( inputs(1:2:end), tag ), 1 );
            if isempty(id)
                id = 0;
            else
                id = id * 2 - 1;
            end
        end
        
        function val = validArgList( varargin )
            % return true if argarg is 'tag1', val1, 'tag2', vale2 ...
            val = false;
            if mod(length( varargin ),2) == 0
                % needs to be even
                for itag = 1 : 2 : length(varargin)-1
                    if ~ischar( varargin{itag} )
                        return;
                    end
                end
                val = true;
            end
        end
        
        function dndr = xy2r( xlist, xyDist, rList, x0, y0 )
            %  function dndr = xy2r( xlist, xyDist, rList, x0, y0 )
            %  Given a density function like dP/dxdy as xyDist( xlist, xlist ), 
            %  dndr = integral of xyDist in r1 < r < r2 / (pi*r2^2 - pi*r1^2)
            %  where r1 and r2 are given as a series in rList
            if exist( 'x0', 'var' )
                if ~exist( 'y0', 'var' )
                    y0 = 0;
                end
                [xg,yg] = meshgrid( xlist, xlist );
                xyDist = interp2( xg, yg, xyDist, xg+x0, yg+y0 );
                xyDist(isnan(xyDist)) = 0;
            end
            
            % dndr(jj) is density around rList(jj), i.e., 
            % integral over the area in radius rBound(jj) to rBound(jj+1)
            % i.e., between rBound(jj) = (rList(jj) + rList(jj-1))/2 and
            %               rBound(jj+1) = (rList(jj+1) + rList(jj))/2
            % rBound(1) = rList(1) - (rList(2)-rList(1))/2 = 1/2( rList(1) + 2*rList(1)-rList(2)
            % rBound(end) = rList(end) + (rList(end)-rList(end-1))/2 = 1/2( rList(end) + 2*rList(end) - rList(end-1))
            rBound = ([2*rList(1)-rList(2), rList] + [rList, 2*rList(end)-rList(end-1)])/2;
            if rBound(1) < 0
                rBound(1) = 0;
            end
            S = zeros( 1, length(rBound) );
            for jj = 1 : length(rBound)
                inArea = SqInCircle( xlist, xlist, rBound(jj), rBound(jj) );
                S(jj) = sum( sum( xyDist .* inArea ) );
            end
            % dndr(jj) = (S(jj+1) - S(jj)) / ( pi*rBound(jj+1)^2 - pi*rBound(jj)^2 );
            % if dndr < 0, make the ciarcle larger until the edge
            dndr = nan( 1, length(rList) );
            for jj = 1 : length(dndr)
                jjup = jj; jjdown = jj+1;
                while jjup < length(S) && jjdown >= 2
                    jjup = jjup + 1;
                    jjdown = jjdown - 1;
                    dS = S(jjup) - S(jjdown);
                    if dS > 0
                        break;
                    end
                end
                if dS > 0
                    dndr(jj) = dS / (pi*(rBound(jjup)+rBound(jjdown))*(rBound(jjup)-rBound(jjdown)));
                end                
            end
        end
        
        function dmap = difAmp( xlist, ampList, rx0, ry0, x0, y0, rx1, ry1, x1, y1 )
            % function ampList = difAmp( xlist, ampList, rx0, ry0, x0, y0, rx1, ry1, x1, y1 )
            %  dmap = ampList in an oval with radius (rx0,ry0) centered at (x0, y0) -
            %         ampList in an oval with radius (rx1,ry1) centered at (x1, y1);
            inArea0 = SqInCircle( xlist-x0, xlist-y0, rx0, ry0 );
            inArea1 = SqInCircle( xlist-x1, xlist-y1, rx1, ry1 );
            % SqInCircle is weight with area included.
            dmap = ampList .* (inArea1 - inArea0) / (xlist(2)-xlist(1))^2;
        end

        function val = sumIn( xlist, ampList, rx, ry, x0, y0 )
            % function val = sumIn( xlist, ampList, rx, ry )
            % sum(sum( ampList.^2) ) * dx^2 within radius (rx,ry)
            % r can be array, so that power in a series of rings can be calculated
            %{ 
                 % validation
                 x = (-99.5:99.5)/100*0.25; [xg,yg] = meshgrid(x,x);
                 r = 0.01:0.01:0.25;
                 % accuracy of area of a circle
                 subplot(2,1,1)
                 plot( r, UtilTK.sumIn( x, ones(size(xg)), r, r) - pi*r.^2 )
                 w=0.062; x0=0.02; y0=-0.01;
                 fld = sqrt(2/pi)/w*exp(-((xg-x0).^2+(yg-y0).^2)/w^2);
                 % accuracy of a shifted gaussian beam
                 subplot(2,1,2)
                 plot( r, UtilTK.sumIn( x, fld, r, r, x0, y0 ) - (1-exp(-2*r.^2/w^2)) )
            %}
            
            if ~exist('rx','var')
                rx = max(abs(xlist)) * sqrt(2);
            end
            if ~exist('ry','var')
                ry = rx;
            end

            if exist( 'x0', 'var' )
                if ~exist( 'y0', 'var' )
                    y0 = 0;
                end
                [xg,yg] = meshgrid( xlist, xlist );
                ampList = interp2( xg, yg, ampList, xg+x0, yg+y0 );
                ampList(isnan(ampList)) = 0;
            end
                        
            datSize = min( length(rx), length(ry) );
            val = zeros( 1, datSize );
            for jj = 1 : datSize
                inArea = SqInCircle( xlist, xlist, rx(jj), ry(jj) );
                val(jj) = sum( sum( abs(ampList).^2 .* inArea ) );
            end
        end
%{      
        sumOut = sumIn( -r );
        function val = sumOut( xlist, ampList, rx, ry ) %, x0, y0 )
            % function val = sumOut( xlist, ampList, rx, ry, x0, y0 )
            % sum(sum( ampList.^2) ) * dx^2 outside of radius (rx,ry) and offcentered at (x0,y0)
            
            if ~exist('rx','var')
                rx = max(abs(xlist));
            end
            if ~exist('ry','var')
                ry = rx;
            end
%             if ~exist('x0','var')
%                 x0 = 0;
%             end
%             if ~exist('y0','var')
%                 y0 = 0;
%             end
            
            % leng = length( xlist );
            datSize = min( length(rx), length(ry) );
            val = zeros( 1, datSize );            
            for jj = 1 : datSize
                inArea = (xlist(2)-xlist(1))^2 - SqInCircle( xlist, xlist, rx(jj), ry(jj) ); % outside = 1, inside = 0
                val(jj) = sum( sum( abs(ampList).^2 .* inArea ) );
            end
        end
%}
        function r2 = XYsq( x, y, x0, y0, rx, ry )
            % function r2 = XYsq( x, y, x0, y0, rx, ry )
            % (x-x0)^2 / rx^2 + (y-y0)^2 / ry^2
            
            if ~exist('ry','var')
                ry = 1;
                if ~exist('rx','var')
                    rx = 1;
                    if ~exist('y0','var')
                        y0 = 0;
                        if ~exist('x0','var')
                            x0 = 0;
                        end
                    end
                end
            end
            
            if size(x,2) == 1
                x = x';
            end
            if size(y,2) == 1
                y = y';
            end
            
            [xg,yg] = meshgrid( x, y );
            r2 = ( xg - x0 ).^2 / rx^2 + (yg - y0).^2 / ry^2;
        end
        
        function r2 = Rsquare( x, x0, y0, rx, ry )
            % function r2 = Rsquare( x, x0, y0, rx, ry )
            % (x-x0)^2 / rx^2 + (x-y0)^2 / ry^2
            
            if ~exist('rx','var')
                rx = 1;
            end
            if ~exist('ry','var')
                ry = 1;
            end
            if ~exist('x0','var')
                x0 = 0;
            end
            if ~exist('y0','var')
                y0 = 0;
            end
            
            if size(x,2) == 1
                x = x';
            end
            
            [xg,yg] = meshgrid( x, x );
            r2 = ( xg - x0 ).^2 / rx^2 + (yg - y0).^2 / ry^2;
            
        end
        
        function mat1 = useQuarter( mat4, Nsmall )
            % mat1 = useQuarter( mat4 )
            % return the central (N/2xN/2) area
            Nlarge = length( mat4 );
            if ~exist( 'Nsmall', 'var' )
                Nsmall = Nlarge/2;
            end
            mat1 = mat4( 1+(Nlarge-Nsmall)/2 : (Nlarge+Nsmall)/2, ...
                1+(Nlarge-Nsmall)/2 : (Nlarge+Nsmall)/2 );
        end
        
        function mat4 = expand2x2( mat1, Nlarge )
            % mat4 = expand2x2( mat1, Nlarge )
            % expand mat1 to NlargexNlarge by filling zeros outside of mat1
            % default Nlarge is 2 x mat1 size
            Nsmall = length( mat1 );
            if ~exist('Nlarge', 'var')
                Nlarge = Nsmall * 2;
            end
            mat4 = zeros( Nlarge, Nlarge );
            mat4( 1+(Nlarge-Nsmall)/2 : (Nlarge+Nsmall)/2, ...
                1+(Nlarge-Nsmall)/2 : (Nlarge+Nsmall)/2 ) = mat1;
        end
        
        function filter = lowpassFilter( Nx, Ny, Fcutoff, Nfilter )
            % filter = lowpassFilter( Nx, Ny, Fcutoff, Nfilter )
            % simple low pass filter
            if Fcutoff <= 0 || Fcutoff > 0.5
                error( 'Fcutoff (%g) must be between (0, 0.5]', Fcutoff );
            end
            if mod(Nfilter,2) ~= 0
                error( 'Nfilter (%g) must be an integer', Nfilter );
            end
            [fx,fy] = meshgrid( ((1:Nx) - fix(Nx/2)-1)/Nx, ((1:Ny) - fix(Ny/2)-1)/Ny );
            %            [fx,fy] = meshgrid( ((1:Nx) - (Nx/2)-0.5)/Nx, ((1:Ny) - (Ny/2)-0.5)/Ny );
            filter = 1 ./ (1 + ((fx.^2 + fy.^2) / Fcutoff^2 ).^Nfilter );
        end
        
        function mapOut = lowPassMap( mapIn, xIn, yIn, xOut, yOut, Fcutoff, Nfilter, tilt, cutHF )
            % function mapOut = lowPassMap( mapIn, xIn, yIn, xOut, yOut, Fcutoff[0.5], Nfilter[20], tilt[0], cutHF[false] )
            %
            % mapFiltered = real( ifft2(ifftshift( filterWgt .* fftshift(fft2(mapIn)))) );
            % mapOut = cos(tilt) * interp2( xIn*cos(tilt), mapFiltered, xOut )
            %
            % apply low-pass butterworth filter on input map, defined in [xIn, yIn]
            % and returns map in [xOut, yOut]
            % cutoff frequency (0~0.5) and order of filter can be specified
            %     filter = 1 ./ (1.0 + (fx^2 + yf^2) ./ Fcutoff^2 ).^(Nfiter))
            %
            % lowpass filter is applied when the new grid size is larger than factor 2
            %
            % tilt is applied so that this map is to be seen by a field coming with angle tilt (radian)
            % i.e., when mapping from [xIn,yIn] to [xOut,yOut], actually xIn*cos(tilt) is used and
            % the mapOut = mapOut*cos(tilt)
            %
            % if cutHF is true, high frequecy component is cut down as :
            % mapF = filterWgt .* fftshift(fft2(mapIn)) is shrink down to the size of (xOut,yOut)
            % which cuts the higest frequency by size of Xout / size of Xin = Fmax out / Fmax in
            %
            
            if size(mapIn,2) ~= length(xIn) || size(mapIn,1) ~= length(yIn)
                error( 'mapIn size ( [%d, %d] ) must match with the size of xIn and yIn', ...
                    size(mapIn), length(xIn), length(yIn) );
            end
            
            % if x and y are in one row or column vector, make x a row vector and y a column vector
            if size(xOut,2) == 1
                xOut = xOut';
            end
            if size(yOut,1) == 1
                yOut = yOut';
            end
            if size(xIn,2) == 1
                xIn = xIn';
            end
            if size(yIn,1) == 1
                yIn = yIn';
            end
            
            if ~exist( 'Fcutoff', 'var' ) || Fcutoff == 0
                Fcutoff = 0.5;
            end
            if ~exist( 'Nfilter', 'var' ) || Nfilter == 0
                Nfilter = 20;
            end
            if ~exist( 'tilt', 'var' )
                tilt = 0;
            end
            if ~exist( 'cutHF', 'var' )
                cutHF = false;
            end
            
            binRatio = min(xIn(1,2)-xIn(1,1),yIn(2,1)-yIn(1,1))/max(xOut(1,2)-xOut(1,1),yOut(2,1)-yOut(1,1));
            
            if binRatio < 0.5
                
                Nfilter = fix( Nfilter );
                
                % apply filter
                NaN_Mask = ~isfinite(mapIn);
                mapIn(NaN_Mask)=0;
                mapTemp = fftshift(fft2(ifftshift(mapIn)));
                %                mapTemp = fftshift(fft2(mapIn));
                
                % cut the high frequency component and use the coarse spacing xOut and yOut
                if cutHF
                    NXin = length(xIn);
                    NYin = length(yIn);
                    
                    % NXout is number of xOut points in the range of xIn, like W = 40cm and mirror data is in 30cm
                    NXout = length( find( xOut(1,:)) >= xIn(1,1) & xOut(1,:) <= xIn(1,end) );
                    NYout = length( find( yOut(:,1)) >= yIn(1,1) & yOut(:,1) <= yIn(end,1) );
                    
                    % When the frequency range size is cut from Nin to Nout without changing the frequency spacing,
                    % the window size W is unchanged, while the grid spacing becomes larger by Nin/Nout.
                    % Out spacing = W/Nin * (Nin/Nout) = W/Nout = spacing of xOut
                    mapTemp = mapTemp( 1+floor((NYin-NYout)/2):floor((NYin+NYout)/2),...
                        1+floor((NXin-NXout)/2):floor((NXin+NXout)/2)) * (NXout*NYout)/(NXin*NYin);
                    
                    xIn = (xIn(1,end)-xIn(1,1))/(2*NXout)*(-NXout+1:2:NXout-1);
                    yIn = (yIn(end,1)-yIn(1,1))/(2*NYout)*(-NYout+1:2:NYout-1);
                end
                % Anti-aliasing filter;
                % calculate filter
                Fcutoff = min( [0.5*binRatio, Fcutoff, 0.5] );
                filterWgt = UtilTK.lowpassFilter( length(xIn), length(yIn), Fcutoff, Nfilter );
                
                mapIn = real( fftshift( ifft2 ( ifftshift(filterWgt .* mapTemp) ) ) );
                %               mapIn = real( ifft2 ( ifftshift(filterWgt .* mapTemp) ) );
                
                if ~cutHF
                    % for frequency cut case, how to restore the NaN information ???
                    mapIn(NaN_Mask) = NaN;
                end
            end
            
            if size(yIn,1) == 1
                yIn = yIn';
            end
            [xIn, yIn] = meshgrid( xIn(1,:)*cos(tilt), yIn(:,1) );
            mapOut = cos(tilt) * interp2(xIn,yIn,mapIn,xOut,yOut);
        end
        
        function [datOut, xOut, yOut] = pickCircle( datIn, xIn, yIn, x0, y0, radius )
            % [datOut, xOut, yOut] = pickCircle( datIn, xIn, yIn, x0, y0, radius )
            % from the datIn at (xIn, yIn), select set data in a circle
            % with center at (x0,y0) within radius
            indx = find( abs(xIn-x0) <= radius );
            if mod(length(indx),2) == 1
                indx = indx(1:end-1);
            end
            indy = find( abs(yIn-y0) <= radius );
            if mod(length(indy),2) == 1
                indy = indy(1:end-1);
            end
            xOut = xIn(indx) - x0;
            yOut = yIn(indy) - y0;
            datOut = datIn(indy, indx);
        end
        
        % weighted average and std, from Rich_ave_std.m of Richard Day
        function [wstd, wave] = stdaveW( val, weight )
            weight = weight(isfinite(val));
            val = val(isfinite(val));
            
            wave = sum(weight.*val)/(sum(weight));
            wstd = sqrt( (sum(weight.*(val - wave).^2))/sum(weight) );
        end
        
        function [argList, endPos] = pickArg( str )
            % function argList = pickArg( str )
            % For a string input '( a, b(), c )', this returns a, b() and c
            argList = cell(0);
            endPos = 0;
            if str(1) ~= '('
                return;
            end
            counter = 0;
            argBeg = 0;
            for ind = 2 : length( str )
                c = str(ind);
                if c == '('
                    counter = counter + 1;
                elseif c == ')'
                    counter = counter - 1;
                    if counter < 0
                        break;
                    end
                else
                    if counter == 0
                        if c == ','
                            if argBeg ~= 0
                                argList{ length(argList)+1 } = str(argBeg:ind-1);
                                argBeg = 0;
                            end
                        else
                            if argBeg == 0
                                argBeg = ind;
                            end
                        end
                    end
                end
            end
            if counter == 0
                ind = length( str ) + 1;
                endPos = length( str );
            else
                endPos = ind;
            end
            if argBeg ~= 0
                argList{ length(argList)+1 } = str(argBeg:ind-1);
            end
            
        end
        
        % ind = wildMatch( strList, pat )
        % simpleReg finds strings in a string call array which matches with patter, which can have * as wildcard
        % This is a simple version of regexp( strList, '|pat( * -> .* )$'
        % example : UtilTK.wildMatch( {llo.fields.fieldList(:).name}, 'BS*i');
        function ind = wildMatch( strList, pat )
            expression = '|';
            for ii = 1 : length(pat)
                if pat(ii) == '*'
                    expression(length(expression)+1) = '.';
                    expression(length(expression)+1) = '*';
                else
                    expression(length(expression)+1) = pat(ii);
                end
            end
            ind = zeros(0);
            expression(length(expression)+1) = '$';
            matched = regexp( strList, expression );
            for ii = 1 : length(matched)
                if ~isempty(matched{ii})
                    ind(length(ind)+1) = ii;
                end
            end
        end
        
        % [x, val] = findMax( xList, valList )
        %    find peak of valList. Values around the peak is used to find the maximal point by polyfit
        function [x, val] = findMax( xList, valList )
            imax = find( valList == max(valList) );
            if ~exist('xList', 'var')
                xList = 1:length(valList);
            end
            if imax == 1
                x = xList(1); val = valList(1);
            elseif imax == length(valList)
                x = xList(end); val = valList(end);
            else
                if imax == 2
                    imax = 3;
                elseif imax == length(valList)-1
                    imax = imax - 1;
                end
                P = polyfit( xList(imax-2:imax+2), valList(imax-2:imax+2), 2 );
                x = -P(2)/(2*P(1));
                val = P(3) - P(2)^2/(4*P(1));
            end
        end
        
        % function Ival = integrate( val, dx, asIs [false] )
        %    Ival = integrate over the first index if asIs is true
        %           if not asIs, integrate over index whose size is larger.
        %           A(100,2) or A(2,100) integrate over 1:100
        function Ival = integrate( val, dx, asIs )
            % integrate over the first integral
            if ~( exist('asIs', 'var') && asIs )
                if size(val,1) < size(val,2)
                    val = val';
                end
            end
            Ndat = size(val,1);
            if mod(Ndat,2) == 0
                Ndat = Ndat - 1;
            end
                
            Ival = zeros(1,size(val,2));
            for ic = 1 : size(val,2)
                 Ival(ic) = dx / 3 * ( val(1,ic) + val(Ndat,ic) ...
                     + 4*sum(val(2:2:Ndat-1,ic)) + 2*sum(val(3:2:Ndat-1,ic)) );                
            end
        end
    end
end

