% manage caviies
%
%
% May 29, 2019
%    pathFromA2B(A,B) is modified to be smarter to find the path more effectively, I hope.
%    fullTree is added as a member data, which is a expanded version of tree. In the future, replace all to fullTree.
%
% September 4th, 2018
%    pathFromA2B(A,B) is modified to return A if A and B are the same.
%
% August 2, 2018
%    new function beam2tilt added. This calculates the tilts of ITM and ETM when the cavity beam axis
%    is defined by the beam positions on ITM and ETM.
%    This is a static function, and all parameters are to be given.
%    function [thetaI,phiI,thetaE,phiE] = beam2tilt( xI, yI, xE, yE, invRoCI, invRoCE, L )
%
% May 18, 2018
%    new functions added
%        function qInvOut = fillCavityInvQ( obj, cavity, qInv, bothWay )
%        function [phaseSum, propList] = kLGouyOfClosedCavity( obj, propID, traceOn )
%
% February 27, 2018
%   function totalLeng = lengthFromA2B( obj, startPort, endPort ) added
%       This caslculates total optical path length between startPort to endPort
%
% February 25, 2018
%   function fullPath = pathFromA2B( obj, startPort, endPort ) added,
%       This returns the full path list from startPort to endPort.
%
% February 23, 2018
%   function [BSID, ITMXID, ITMYID] = isMichelson( obj, BSname ) added.
%   If there is BS-ARx <=> ITMX, and BS-HRy <=> ITMY, BSID > 0.
%
% February 8, 2018
%  1) function phaseSum = totalLockPhaseOfClosedCavity( obj, propID ) added
%     calculates sum of locked phases in a closed loop
%  2) totalPhaseOfClosedCavity does not subtract lockPhase
%     This is renamed to kLGouyPhase.
%
% February 7, 2018
% 1: function [phaseSum, propList] = totalPhaseOfClosedCavity( propID ) added
%   For a closed cavity which has the propagator propID as a part of it,
%   propList(1:2) = [propID, propObj.totalPhase(propID, true)] is returned
%   phaseSum = sum(propList(2,:))
% 2: function cID = findClosedCavID( obj, id ) modified to accept propID as well as portID
%   if id > 0 or id is a string, it is port ID
%   if id < 0, it is propID
%
% March 31, 2017
%   function [gouyList, propIDs] = findCavityGouy( obj, cavity, q )
%   new function to calculate gouy phase at each propagation defined in cavity
%   cavity = ifo.cavities.closedCavs{N}
%
% September 18, 2015
%   function name changed from propFields to propFieldsOnPath
%
% October 27, 2014
%   markUnusedFields is added, which sets the status of field to be -1 if the field is not in the tree
%   fields.calcIfNecessary stops tracing backk if the status is non-zero, so this -1 marks the stop.
%
% October 26, 2014
%   handle of FFTIFO itself is saved in all manager objects,
%   instead of handle of multiple manager handles
%
% October 23, 2014
%   debug object is added, which is passed to GuessObj
%
% October 22, 2014
%   setQvals, called from buildTree, had a serious bug, when fining not-yet-qInv-filled fields,
%   the code was ~isnan, but it should have been isnan. Fortunately, they were filled
%   somewhere else in a brute force way.
%
% October 14, 2014
%   In buildGuessList, if there is a back scattering mirror are in two closed cavities,
%   first one is NOT onTheFly and the second one'sconvergence error is 10 times loose.
%   This is for Virgo MC with MC2 backscattering.
%
% August 19, 2014
%   easyLock and calcLoopPhase removed. They are in GuessObj now.
%

classdef cavityManager < handle
    
    % these are not filled by any of the built in functions
    % these are used printAllInfo
    properties
        ifo
        
        tree        % original compact tree for a given root  A -> B -> null; C -> A
        fullTree    % full tree, ends only unattached branch or self. A -> B -> null; C -> A -> B -> null, used in pathFromA2B
        closedCavs
        
        lengthFreezeFlag = false;
        
        compList  % compound system list, for now HR loop, like FP, 3 and 4 mirror MC
        guessList % systems needed to calculate guess fields
        
        pathBook
    end
    
    methods
        % constructors
        function obj = cavityManager( ifo )
            obj.ifo = ifo;
            
            obj.compList = CompObj.empty(0);
            obj.guessList = GuessObj.empty(0);
        end
        
        % if all guess are caluclated
        function ready = loopFieldReady( obj )
            ready = false;
            for iguess = 1 : length( obj.guessList )-1
                % not yet calculated or will never be calculated
                if obj.ifo.fields.fieldList( obj.guessList(iguess).guessID ).status <= 0
                    return;
                end
            end
            ready = true;
        end
        
        % return pass list (list of out going ports) connecting start port to end port.
        % E.g., begin = PRM-HR-o, end = ITMY-HR-o, the path will be {'PRM-HR-o', 'BS-HRy-o', 'ITMY-HR-o'}
        % for now, only when those paths are recognized when startPort is closer to the source than endPort
        function fullPath = pathFromA2B( obj, varargin )
            if nargin == 1
                fullPath = [];
            elseif nargin == 2
                fullPath = varargin{1};
            elseif nargin == 3
                startPort = obj.ifo.fields.findNamedField(varargin{1});
                endPort = obj.ifo.fields.findNamedField(varargin{2});
                if startPort == endPort
                    fullPath = startPort;
                else
                    fullPath = {};
                    for it = 1:length(obj.fullTree)
                        iStart = find( obj.fullTree{it} == startPort, 1 );
                        if ~isempty( iStart )
                            iEnd = find( obj.fullTree{it}(iStart+1:end) == endPort );
                            if ~isempty( iEnd )
                                path = obj.fullTree{it}(iStart:iStart+iEnd);
                                if isempty(fullPath) || length(fullPath) > length(path)
                                    fullPath = path;
                                end
                            end
                        end
                    end
                end
            else
                fullPath = obj.pathFromA2B(varargin{1}, varargin{2});
                fullPath = [fullPath(1:end-1), obj.pathFromA2B(varargin{2:end})];
            end
        end
        
        % total optical lengths from startPort to endPort
        % example : start = 'PRM-HR-o', and end = 'ITM-HR-o'
        function totalLeng = lengthFromA2B( obj, startPort, endPort, traceOn )
            pathList = obj.pathFromA2B( startPort, endPort );
            totalLeng = 0;
            for ip = 1 : length(pathList)-1
                [sinkID, propID] = obj.ifo.props.getPropTo( pathList(ip), true );
                if propID > 0
                    if obj.ifo.optics.findOptic( sinkID ) == obj.ifo.optics.findOptic( pathList(ip+1) )
                        thisLeng = obj.ifo.props.propList(propID).L0 ...
                            + obj.ifo.optics.getDeapth( sinkID, 1 ) ...
                            + obj.ifo.optics.getDeapth( pathList(ip+1), 1 );
                        totalLeng = totalLeng + thisLeng;
                        if exist('traceOn', 'var') && traceOn
                            fprintf(1, '  > leng of %s = L0( %e ) + n opt( %e ) = %e\n', ...
                                obj.ifo.props.propList(propID).name, ...
                                obj.ifo.props.propList(propID).L0, ...
                                obj.ifo.optics.getDeapth( sinkID, 1 ) ...
                                + obj.ifo.optics.getDeapth( pathList(ip+1), 1 ), thisLeng);
                        end
                    end
                end
            end
        end
        
        % Check if the IFO has BS which is connected with ITMX and with ITMY
        function [BSID, ITMXID, ITMYID] = isMichelson( obj, BSname )
            if ~exist('BSName', 'var')
                BSname = 'BS';
            end
            
            BSID = obj.ifo.optics.findNamedOptic( BSname );
            ITMXID = 0;
            ITMYID = 0;
            % BS found
            if BSID > 0
                % field from BS toward X direction
                BSARXID = obj.ifo.fields.findNamedField( strcat( BSname, '-ARx-o' ) );
                % if the field exist, find connection to ITM in X direction
                if BSARXID > 0
                    ITMXARID = obj.ifo.props.getPropTo( BSARXID, true );
                    % there is a connection to ITM in x direcction
                    if ITMXARID > 0
                        % now toward Y direction
                        BSHRYID = obj.ifo.fields.findNamedField( strcat( BSname, '-HRy-o' ) );
                        % field found toward Y direction
                        if BSHRYID > 0
                            ITMYARID = obj.ifo.props.getPropTo( BSHRYID, true );
                            % there is a connection to ITM in x direcction
                            if ITMYARID > 0
                                % now, BS => ITMX and BS => ITMY are found, so it is Michelson
                                ITMXID = obj.ifo.optics.findOptic( ITMXARID );
                                ITMYID = obj.ifo.optics.findOptic( ITMYARID );
                            end
                        end
                    end
                end
            end
            if ITMXID == 0 || ITMYID == 0
                BSID = 0;
            end
        end
        
        function resetFieldStatus( obj )
            for id = 1 : length( obj.ifo.fields.fieldList )
                obj.ifo.fields.fieldList(id).status = 0;
            end
            obj.markUnusedFields;
        end
        
        function updateFieldStatus( obj )
            % mark fields which has been calculated by loop
            for ilp = 1 : length(obj.closedCavs)
                thisLoop = obj.closedCavs{ilp};
                leng = find( thisLoop(2:end) == thisLoop(1), 1 );
                if isempty(leng)
                    leng = length( thisLoop );
                end
                for iele = 1 : leng
                    obj.ifo.fields.fieldList(thisLoop(iele)).status = ilp;
                    obj.ifo.fields.fieldList(obj.ifo.props.getPropTo(thisLoop(iele))).status = ilp;
                end
            end
        end
        
        % change the lengthFreezeFlag to doIt and return the current value
        function current = freezeLength( obj, freezeIt )
            current = obj.lengthFreezeFlag;
            if exist( 'freezeIt', 'var' )
                obj.lengthFreezeFlag = freezeIt;
            end
        end
        
        function clearAllTemp( obj )
            for ig = 1 : length( obj.guessList )
                obj.guessList(ig).clearTempFields;
            end
        end
        
        % one of the main calls from FFTIFO to analyze the IFO structure
        % this does not use Nfft/Wfft information
        % those, which use Nfft/Wfft, are in setupMisc
        function analyzeCavity( obj, root )
            % build tree
            obj.tree = obj.buildTree( root );
            obj.fullTree = obj.buildTree( -root );
            % fund closed cavities, which are used for mode definitions and for the guessField search
            obj.closedCavs = obj.findClosedCavs( obj.tree );
        end
        
        % these need FFT information
        function setupMisc( obj )
            % compound optics, may not be necessary
            obj.buildCompList;
            % build optics systems which are used to calculate guess fields
            obj.buildGuessList;
        end
        
        function [qInv, root] = findModeBaseByClosedCavs( obj )
            root = 0;
            % find a loop formed by HR reflections, like FP or MC with 3 or 4 mirrors
            for icav = length( obj.closedCavs ) : -1 : 1
                if obj.isHRloop( obj.closedCavs{icav} )
                    [qInv, root] = obj.findCavityMode( obj.closedCavs{icav} );
                    break;
                end
            end
            if root == 0
                % no FP in the IFO, so use the last cavity
                [qInv, root] = obj.findCavityMode( obj.closedCavs{end} );
            end
        end
        
        function buildGuessList( obj )
            % this is in charge of providing external pump field
            for ic = 1 : length( obj.closedCavs )
                obj.guessList(ic) = GuessObj( '', ic, obj.ifo );
            end
            
            % like MC with back scattering, if two closed cavities are composed of same optics
            % first closedLoop is not onTheFly, otherwise, infinite loop may happen,
            % like Virgo MC
            [backScat1, backScat2] = obj.findBackScatteredCavity;
            if backScat1 > 0 && backScat2 > 0
                obj.guessList(backScat1).onTheFly = false;
                obj.guessList(backScat2).errorLimit = obj.guessList(backScat2).errorLimit * 100;
            end
            
            % this is a special to provide pump sources which are not provided by any closed loop, like laser source
            obj.guessList( length( obj.closedCavs ) + 1 ) = ...
                GuessObj( 'MasterGuess', 0, obj.ifo );
            
            % all tree elements added to relevant guess loop
            % tree finder and this section may need to be improved
            for it = 1 : length( obj.tree )
                if obj.tree{it}(end) > 0
                    % pick up the external source
                    aTree = obj.tree{it};
                    for in = 2 : length( aTree )
                        % first element which belongs to a specific closed loop cavity
                        firstGuess = obj.findClosedCavID( aTree(in) );
                        if firstGuess > 0
                            % we found the first loop entry
                            % this is the source to the first entry point of the closed loop cavity
                            externalSrc = aTree(in-1);
                            % from here to the end is the section to be analyzed
                            aTree = aTree(in:end);
                            break;
                        end
                    end
                    guessTable = obj.analyzeATree( aTree );
                    % first is the source to the first loop element
                    % so the source is marked by -source, which signified the change of level
                    guessTable{firstGuess} = [-externalSrc, guessTable{firstGuess} ];
                    % add path to each guess field loop
                    for ig = 1 : length( obj.closedCavs )
                        obj.guessList(ig).addPath( guessTable{ig} );
                    end
                end
            end
            % build master path so that the field evolution can be calculated in one path
            for ig = 1 : length( obj.closedCavs )
                obj.guessList(ig).buildMasterPath;
            end
            
            % if the first one is onTheFly, it is FP or MC, so make it locked
            if obj.guessList(1).onTheFly
                obj.guessList(1).errorLimit = 1e-6;
                obj.guessList(1).Nacc = 5;
                obj.guessList(1).Nave = 3;
            end
            
            % default error of onTheFly cavity, like arm FP, is large - just one trip calculation
            % !!!!
            % !!!! This should be a cavity which is not connected to laser, FIX IT
            % !!!!
            for ig = 2 : length(obj.guessList)-1
                if obj.guessList(ig).onTheFly
                    obj.guessList(ig).Nacc = 3;
                    obj.guessList(ig).errorLimit = 10;
                end
            end
            
        end
        
        function [backScat1, backScat2] = findBackScatteredCavity( obj )
            backScat1 = 0;
            backScat2 = 0;
            for icav = 1 : length( obj.closedCavs )
                for iopt = 1 : length( obj.closedCavs{icav} )
                    optID = obj.ifo.optics.findOptic( obj.closedCavs{icav}(iopt) );
                    if optID > 0
                        if obj.ifo.optics.optList(optID).backScatter
                            if backScat1 == 0
                                backScat1 = icav;
                                break;
                            elseif backScat2 == 0
                                backScat2 = icav;
                                return;
                            end
                        end
                    end
                end
            end
        end
        
        function guessTable = analyzeATree( obj, aTree )
            curLevel = obj.findClosedCavID( aTree(1) );
            guessTable = cell( length( obj.closedCavs ), 1 );
            while ~isempty( aTree )
                origLeng = length( aTree );
                for in = 1 : origLeng
                    cID = obj.findClosedCavID( aTree(in) );
                    % closed loop is expanded to include branches
                    % make sure it is OK that closed nature is not used any more
                    % now begin and end can be different
                    if cID == 0
                        obj.closedCavs{curLevel} = [obj.closedCavs{curLevel}, aTree(in)];
                    end
                    if cID ~= 0 && cID ~= curLevel
                        % level changed, i.e., field goes into other cavity
                        % add current segment to curLevel
                        guessTable{curLevel} = [guessTable{curLevel}, aTree(1:in-1)];
                        % mark the new level by -field ID of the previous level
                        guessTable{cID} = [guessTable{cID}, -guessTable{curLevel}(end), aTree(in)];
                        % mark the last of the current level by -field ID in the new level
                        guessTable{curLevel} = [guessTable{curLevel}, -aTree(in)];
                        % now the new level becomes the current level
                        curLevel = cID;
                        % aTree is shrunk to the rest
                        aTree = aTree(in+1:end);
                        break;
                    end
                end
                if length(aTree) == origLeng
                    % there was no change of levels at all, i.e., simple one loop
                    guessTable{curLevel} = [guessTable{curLevel}, aTree(1:end)];
                    break;
                end
            end
        end
        
        % sum of locked phase in a closed loop
        function phaseSum = totalLockPhaseOfClosedCavity( obj, propID )
            cID = obj.findClosedCavID( -propID );     % use propID to find close loop
            portList = cell2mat( obj.ifo.cavities.closedCavs( cID ) ); % ports forming the closed cavity
            nProp = length(portList);
            if portList(1) == portList(end)
                nProp = nProp - 1;
            end
            propList = zeros(2,nProp);  % props forming the cavity, last one the same as the start
            for ip = 1 : nProp
                [~,propList(1,ip)] = obj.ifo.props.getPropTo( portList(ip) );
                propList(2,ip) = obj.ifo.props.propList(propList(1,ip)).lockPhase;
            end
            phaseSum = sum( propList(2,:) );
        end
        
        %   For a closed cavity which has the propagator propID as a part of it,
        %   propList(1:2) = [propID, propObj.totalPhase(propID, true)] is returned
        %   phaseSum = sum(propList(2,:))
        %   lockPhase is not subtracted
        function [phaseSum, propList] = kLGouyOfClosedCavity( obj, propID, traceOn )
            if ~exist( 'traceOn', 'var' )
                traceOn = false;
            end
            cID = obj.findClosedCavID( -propID );     % use propID to find close loop
            portList = cell2mat( obj.ifo.cavities.closedCavs( cID ) ); % ports forming the closed cavity
            nProp = length(portList);
            if portList(1) == portList(end)
                nProp = nProp - 1;
            end
            propList = zeros(2,nProp);  % props forming the cavity, last one the same as the start
            for ip = 1 : nProp
                [~,propList(1,ip)] = obj.ifo.props.getPropTo( portList(ip) );
                propList(2,ip) = obj.ifo.props.kLGouyPhase( propList(1,ip), traceOn );
            end
            phaseSum = sum( propList(2,:) );
        end
        
        % find the closed cavityt index where this port id belogs to
        % if id > 0 or id is a string, it is port ID
        % if id < 0, it is propID
        function cID = findClosedCavID( obj, id )
            if id < 0
                id = obj.ifo.props.propList(-id).inPortID;
            else
                id = obj.ifo.fields.findNamedField( id );
            end
            cID = 0;
            for icav = 1 : length( obj.closedCavs )
                if ~isempty( find( obj.closedCavs{icav} == id, 1 ) )
                    cID = icav;
                    break;
                end
            end
        end
        
        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        %
        % Starting from a field on outgoing port "root", find all paths of field propagation
        % This includes paths to unconnected output port
        %
        % One path ends at an unattached port or some node which is already in other branch paths
        %
        % tree is a cell array, each element tree{itree} is a list of fields on outgoing ports
        %
        % buildTree can be used to do actions at each branch by specifying a function handle funcH
        %
        % This does not modify any member data, except funcH might does
        %
        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        function treeOut = buildTree( obj, root, funcH )
            
            compactTree = root > 0;
            root = abs(root);
            
            % empty tree
            treeOut = {};
            
            % current field propagation path starting from root
            % branchPath can grow (when branch grows out) and
            % shrink (trace back when the end of a branch is reached)
            branchPath = root;
            
            % trace goes along reflection branch if available, and transmission branches are stored in trBranch
            % At the end of the reflection path, the latest transmission branch is retrieved
            trBranch = repmat(struct('brIndex',0, 'trNode', 0), 0 );
            
            % go through until the field comes back to the origin after handling all branches
            while ~isempty( branchPath )
                
                % analyze the end of a branch
                branch = branchPath(end);
                
                % this is where reflection and transmission occur
                % If the end is detector or source, that is the end, so require END IS MIRROR
                bud = obj.ifo.props.getPropTo( branch, true );
                
                callFunc = true;
                if bud <= 0 % < 0 is attached to detector
                    % terminate the tree, keep the path and move back
                    treeOut = obj.addBranch( [branchPath, bud], treeOut );
                    [branchPath, trBranch] = obj.back2TrBranch( branchPath, trBranch );
                    
                else
                    % this branch node is connected to something, so expand
                    
                    if ~isempty(find( branchPath(1:end-1) == branch, 1 )) || (compactTree && ~isempty(find( [treeOut{:}] == branch, 1 )))
                        % if the new branch is in any of the known path, register the branch and go to next
                        treeOut = obj.addBranch( branchPath, treeOut );
                        [branchPath, trBranch] = obj.back2TrBranch( branchPath, trBranch );
                        
                        % this branch has been handled already
                        callFunc = false;
                    else
                        % the new branch is to be handled
                        % first add the tr to trBranch for later retrieval
                        trBranch = [ trBranch, struct( 'brIndex', length(branchPath), 'trNode', obj.ifo.optics.findTransPort(bud) ) ];
                        % add the reflection to branchPath for the immediate analysis
                        branchPath = [ branchPath, obj.ifo.optics.findReflPort(bud) ];
                    end
                end
                
                % if function handle is given, call it with the branch as the argument
                if callFunc && exist('funcH', 'var')
                    funcH( branch );
                end
                
            end
        end
        
        % find guess fields and connections of guess fields, used for R_getguessFin
        % guessOut = cell{ number of guess fields }
        %    guessOut{N} = { guessField ID, { loop from field N->1, loop from N->2, ..., loop from N -> last guess field } }
        function guessOut = buildGuessTrees( obj, root, funcH )
            %{
            % empty tree
            treeOut = {};
            
            % current field propagation path starting from root
            % branchPath can grow (when branch grows out) and
            % shrink (trace back when the end of a branch is reached)
            branchPath = root;
            
            % trace goes along reflection branch if available, and transmission branches are stored in trBranch
            % At the end of the reflection path, the latest transmission branch is retrieved
            trBranch = repmat(struct('brIndex',0, 'trNode', 0), 0 );
            
            % go through until the field comes back to the origin after handling all branches
            while ~isempty( branchPath )
                
                % analyze the end of a branch
                branch = branchPath(end);
                
                % this is where reflection and transmission occur
                % If the end is detector or source, that is the end, so require END IS MIRROR
                bud = obj.ifo.props.getPropTo( branch, true );
                
                callFunc = true;
                if bud <= 0 % < 0 is attached to detector
                    % terminate the tree, keep the path and move back
                    treeOut = obj.addBranch( [branchPath, bud], treeOut );
                    [branchPath, trBranch] = obj.back2TrBranch( branchPath, trBranch );
                    
                else
                    % this branch node is connected to something, so expand
                    
                    if ~isempty(find( branchPath(1:end-1) == branch, 1 )) || ~isempty(find( [treeOut{:}] == branch, 1 ))
                        % if the new branch is in any of the known path, register the branch and go to next
                        treeOut = obj.addBranch( branchPath, treeOut );
                        [branchPath, trBranch] = obj.back2TrBranch( branchPath, trBranch );
                        
                        % this branch has been handled already
                        callFunc = false;
                    else
                        % the new branch is to be handled
                        % first add the tr to trBranch for later retrieval
                        trBranch = [ trBranch, struct( 'brIndex', length(branchPath), 'trNode', obj.ifo.optics.findTransPort(bud) ) ];
                        % add the reflection to branchPath for the immediate analysis
                        branchPath = [ branchPath, obj.ifo.optics.findReflPort(bud) ];
                    end
                end
                
                % if function handle is given, call it with the branch as the argument
                if callFunc && exist('funcH', 'var')
                    funcH( branch );
                end
                
            end
            %}
        end
        
        % add a path as a compound optic
        % test if it is a closed loop all facing HR side, like FP or trianular cavity
        function compID = buildCompList( obj )
            for il = 1:length( obj.closedCavs )
                path = obj.closedCavs{il};
                if obj.isHRloop( path )
                    compID = length( obj.compList ) + 1;
                    bookID = obj.readPathBook( path );
                    obj.compList(compID) = ...
                        CompObj( obj.nameComp( path ), bookID, obj );
                end
            end
        end
        
        % find the optic to which portID is attached
        % if portID is an output port, find
        %       the input portID which transmisstes to the output port and
        %       the output port this input port reflects to
        % if portID is an input port, find
        %       the output port to which the input port transmits to and
        %       the the input port which reflects to this output port
        % If the ports are connected, return positive port ID, negative if unconnected
        %
        function [inPort, outPort] = findConnectedInOutPort( obj, portID )
            [inPort, outPort] = obj.ifo.optics.findInOutPort( portID );
            if obj.ifo.props.getPropID( outPort ) == 0
                outPort = 0;
            end
            if obj.ifo.props.getPropFrom( inPort ) <= 0
                inPort = 0;
            end
        end
        
        function name = nameComp( obj, path )
            switch length( path )
                case 3
                    name = 'FP';
                case 4
                    name = 'TriCav';
                case 5
                    name = 'QuadCav';
                otherwise
                    name = fprintf('%dMirCav', length(path)-1 );
            end
            sameNames = strfind([obj.compList(:).name],name);
            name = sprintf('%s%d', name, length(sameNames)+1 );
        end
        
        % propagate field following path
        function propFieldsOnPath( obj, path )
            % look for cache if this has been analized, if not, add one
            findMatch = obj.readPathBook( path );
            for istep = 1 : length( obj.pathBook{2, findMatch} )
                action = obj.pathBook{2, findMatch}(istep).action;
                switch action
                    case 1   % propagation
                        obj.ifo.props.propField( obj.pathBook{2, findMatch}(istep).id );
                    case 2   % reflection
                        obj.ifo.optics.refl( obj.pathBook{2, findMatch}(istep).srcID );
                    case 3   % transmission
                        obj.ifo.optics.trans( obj.pathBook{2, findMatch}(istep).srcID );
                end
            end
        end
        
        function id = readPathBook( obj, path )
            id = 0;
            for idd = 1:size( obj.pathBook, 2 )
                if UtilTK.areSameVecs( path, obj.pathBook{1,idd} )
                    id = idd;
                    return;
                end
            end
            if id == 0
                id = size( obj.pathBook, 2 ) + 1;
                obj.pathBook{1,id} = path;
                obj.pathBook{2,id} = obj.analyzePath( path );
            end
        end
        
        % path = array of out going fields, may be terminated by -sink ID
        % path is converted to
        %    src field ID
        %    action : (1) propagation or
        %             (2) reflection or
        %             (3) transmision
        %       prop : propID
        %       reflection : optic ID, inPort index
        %       transmission : optic ID, inport index
        %    sink field ID
        %
        function pathRec = analyzePath( obj, path )
            pathRec = repmat( struct('srcID', 0, 'action', 0, 'id', 0, 'index', 0, 'sinkID', 0), 1, 0 );
            for ip = 1 : length( path )
                if path(ip) > 0
                    if (ip > 1) && ~isempty( find( path(1:(ip-1)) == path(ip), 1) )
                        % if this has already been handled, this is a closed loop
                        break;
                    end
                    % this is source of a propagator
                    % find where it goes to and with what prop
                    [sinkID, propID] = obj.ifo.props.getPropTo( path(ip) );
                    if propID <= 0
                        break;
                    end
                    % add one branch
                    pathID = length( pathRec ) + 1;
                    pathRec(pathID).srcID = path(ip);
                    pathRec(pathID).action = 1;
                    pathRec(pathID).id = propID;
                    pathRec(pathID).sinkID = sinkID;
                    
                    % find next action, from sinkID to path(ip+1)
                    if ip == length(path)
                        break;
                    end
                    if path(ip+1) < 0
                        break;
                    end
                    % there is something out there
                    [optID, portIndex] = obj.ifo.optics.findOptic( sinkID );
                    reflID = obj.ifo.optics.findReflPort( sinkID );
                    transID = obj.ifo.optics.findTransPort( sinkID );
                    if (reflID == path(ip+1)) || (transID == path(ip+1))
                        pathID = length( pathRec ) + 1;
                        pathRec(pathID).srcID = sinkID;
                        if reflID == path(ip+1)
                            pathRec(pathID).action = 2;
                        else
                            pathRec(pathID).action = 3;
                        end
                        pathRec(pathID).id = optID;
                        pathRec(pathID).index = portIndex;
                        pathRec(pathID).sinkID = path(ip+1);
                    else
                        break;
                    end
                end
            end
        end
        
        % loop is connecting only HR surfaces, like FP or MC cavity
        function itIs = isHRloop( obj, path )
            itIs = false;
            if path(1) == path(end)
                % make sure it is a closed loop
                pathID = obj.readPathBook( path );
                for istep = 1 : length( obj.pathBook{2, pathID} )
                    switch obj.pathBook{2, pathID}(istep).action
                        case 1   % propagation
                            if ~( obj.ifo.optics.isHRside( obj.pathBook{2, pathID}(istep).srcID ) && ...
                                    obj.ifo.optics.isHRside( obj.pathBook{2, pathID}(istep).sinkID ) )
                                % start or end is not on HR side
                                return;
                            end
                        case 2   % reflection, it is good
                            continue;
                        case 3   % transmission, it is bad, either is AR
                            return;
                    end
                end
                itIs = true;
            end
        end
        
        % For a cavity defined by [port1, port2, ..., portN ],
        % propagation length from portAdj to portAdj+1 is modified so that
        % the field with q1 at port1 will have the curvature on the optic to which the field from portN arrives.
        % Example
        %   SCC, [RM, ITM, ETM], cavity = [ITM ETM] will be out from ITM AR and q1 is defined by the eigenstate of ITM-ETM
        %   PRC, [PRM PR2 PR3 BS ITM ETM], cavity = [ITM AR, BS HRx, PR3 HRx, PR2 HRx], q1 is defined by ITM-ETM and
        %        portAdj = PRM3 HRx
        % if modifyL0 is true, corresponding propagators are modified
        function Ladj = adjustModeMatch( obj, qInvIn, portAdj, dl, cavity, modifyL0, accuracy )
            if ~exist( 'modifyL0', 'var' )
                modifyL0 = true;
            end
            if ~exist( 'accuracy', 'var' )
                accuracy = 1e-4;
            end
            % last propagation
            sinkID = obj.ifo.props.getPropTo( cavity(end) );
            % this is the RoC of the field at the end
            invRoC = obj.ifo.optics.findInvRoCHR(sinkID);
            
            % find the propagator to adjust
            propID = obj.ifo.props.getPropID( portAdj );
            % nRef of the part to be adjusted
            nRef = obj.ifo.props.propList( propID ).nRef;
            % this is the length to adjust.
            L0 = obj.ifo.props.getPropLeng( propID );
            if dl == 0
                dl = L0 * 1e-3;
            end
            % to limit the drifting away, limit is set to 1% of the cavity length
            LadjLimit = L0 * 0.01;
            
            Ladj = 0;
            % look for length to make qOut to match with RoC1
            
            M = {eye(2) eye(2)};
            mid = 1;
            for icav = 1:length(cavity)
                if cavity(icav) == portAdj
                    mid = 2;
                end
                [sinkID, propID] = obj.ifo.props.getPropTo( cavity(icav) );
                M{mid} = obj.ifo.props.propList( propID ).propABCD * M{mid};
                if icav < length(cavity)
                    % except for the last one, do the interaction with optics
                    M{mid} = obj.ifo.optics.findABCD( sinkID, cavity(icav+1) ) * M{mid};
                end
            end
            
            dinvR = zeros(1,3);
            for iloop = 1:100
                % calculate propagation with additional -+ dl think propagation
                for iLen = 1:3
                    qInvOut = MMTK.ABCD_qinv( M{2} * MMTK.Prop_ABCD( Ladj + (iLen-2)*dl, nRef ) * M{1}, qInvIn );
                    invRoCOut = real(qInvOut);
                    dinvR(iLen) = invRoCOut - invRoC;
                end
                
                % two RoCs at Ladj +- dl
                dinvRPV = max(dinvR) - min(dinvR);
                % dR/R is small enough
                relErr = abs( dinvRPV / invRoC );
                if relErr < accuracy
                    break;
                end
                
                C2 = (dinvR(3) + dinvR(1)-2*dinvR(2)) / 2;
                C1 = (dinvR(3) - dinvR(1)) / 2;
                C0 = dinvR(2);
                inRoot = C1^2 - 4*C2*C0;
                if inRoot < 0
                    Ladj = NaN;
                    break;
                end
                inRoot = sqrt( inRoot );
                x1 = (-C1 + inRoot)/(2*C2) * dl;
                x2 = (-C1 - inRoot)/(2*C2) * dl;
                
                if abs(x1-Ladj) > abs(x2-Ladj)
                    x1 = x2;
                end
                inLimit = @(a, aLimit) min( max( a, -aLimit ), aLimit );
                dl = inLimit( x1, LadjLimit/10 );
                Ladj = inLimit( Ladj + dl, LadjLimit );
                dl = abs(dl);
            end
            
            if relErr >= accuracy
                Ladj = NaN;
            else
                if modifyL0
                    % if prop length needs to be modified
                    propID = obj.ifo.props.getPropID( portAdj );
                    obj.ifo.props.propList(propID).L0 = obj.ifo.props.propList(propID).L0 + Ladj;
                    % find propagator going backward
                    propID = obj.ifo.props.findReverseProp( propID );
                    if propID > 0
                        obj.ifo.props.propList(propID).L0 = obj.ifo.props.propList(propID).L0 + Ladj;
                    end
                end
            end
        end
        
        % cavity = [out1, out2, ..., outN, out1]
        function [qList, portIDs] = findCavityQ( obj, cavity, q )
            leng = length( cavity ) - 1;
            qList = zeros(1,2*leng);
            portIDs = zeros(1,2*leng);
            qList(1) = q;
            for iport = 1:leng
                portIDs(2*iport-1) = cavity(iport);
                [portIDs(2*iport), propID] = obj.ifo.props.getPropTo( cavity(iport) );
                L0 = obj.ifo.props.getPropLeng( propID );
                qList(2*iport) = qList(2*iport-1) + L0;
                if iport < leng
                    qList(2*iport+1) = MMTK.ABCD_q( obj.ifo.optics.findABCD( portIDs(2*iport), cavity(iport+1) ), qList(2*iport) );
                end
            end
        end
        
        function [gouyList, propIDs] = findCavityGouy( obj, cavity, q )
            leng = length( cavity ) - 1;
            gouyList = zeros(1,leng);
            propIDs = zeros(1,leng);
            for iport = 1:leng
                [sinkID, propIDs(iport)] = obj.ifo.props.getPropTo( cavity(iport) );
                L0 = obj.ifo.props.getPropLeng( propIDs(iport) );
                gouyList(iport) = MMTK.gouyL( q, L0 );
                q = MMTK.ABCD_q( obj.ifo.optics.findABCD( sinkID, cavity(iport+1) ), q + L0 );
            end
        end
        
        % invQ is filled along the path defined by cavity
        %  | =>  P    => | =>
        %  | <=  rP   <= |
        %
        %       -P    => | =>
        %       rP    <= |
        %
        function qInvOut = fillCavityInvQ( obj, cavity, qInv, bothWay )
            % default is both way
            if ~exist( 'bothWay', 'var' )
                bothWay = true;
            end
            % convert cavity input into an array of field IDs
            cavity = obj.ifo.fields.conv2FieldIDList( cavity );
            leng = length( cavity );
            for iport = 1:leng-1
                [sinkID, propID] = obj.ifo.props.getPropTo( cavity(iport) );
                if bothWay
                    revID = obj.ifo.props.findReverseProp( abs(propID) );
                else
                    revID = 0;
                end
                
                % cavity(iport) is an output from an optic ('ITM-HR-o')
                if propID > 0
                    obj.ifo.fields.setQInv( cavity(iport), qInv );
                    % if bothway, fill port going reverse direction
                    if revID > 0
                        obj.ifo.fields.setQInv( obj.ifo.props.propList(revID).outPortID, qInv, true );
                    end
                    
                    % propagate field to the end
                    qInv = MMTK.ABCD_qinv( obj.ifo.props.propList( propID ).propABCD, qInv );
                    
                    % when cavity(iport) is 'ITM-HR-o', sinkID is 'ETM-HR-i
                else
                    % when cavity(iport) os 'ETM-HR-i', set sinkID to 'ETM-HR-i'
                    sinkID = cavity(iport);
                end
                obj.ifo.fields.setQInv( sinkID, qInv );
                if revID > 0
                    obj.ifo.fields.setQInv( obj.ifo.props.propList(revID).inPortID, qInv, true );
                end
                
                % if sinkID == cavity(iport+1), ABCD = 1, so OK to do
                qInv = MMTK.ABCD_qinv( obj.ifo.optics.findABCD( sinkID, cavity(iport+1) ), qInv );
            end
            qInvOut = qInv;
        end
        
        % for cavity = (port1, ..., portN), ABCD matrix is built which combines propagations and reflection or transmission
        % If port1 is an input port, reflection or transmission from port1 to port2, then port2 to ...
        % If portN is an end of the ABCD matrix, i.e., if portN is an input, last will be propagation from portN-1 to portN
        % if portN is an output, last will be a reflection or transmission to portN
        function M = findCavityABCD( obj, cavity )
            cavity = obj.ifo.fields.conv2FieldIDList( cavity );
            % first is assumed to be an out port
            M = eye(2,2);
            leng = length( cavity );
            for iport = 1:leng-1
                [sinkID, propID] = obj.ifo.props.getPropTo( cavity(iport) );
                if propID < 0 % input port, so do reflection or transmission
                    if iport == 1
                        M = obj.ifo.optics.findABCD( cavity(iport), cavity(iport+1) ) * M;
                    else
                        break;
                    end
                elseif propID <= 0
                    break;
                else
                    M = obj.ifo.optics.findABCD( sinkID, cavity(iport+1) ) * obj.ifo.props.propList( propID ).propABCD * M;
                end
            end
        end
        
        % for a series of fields forming a closed loop, an eigen state is calculated
        % The input cavity can be one of cavities in ifo.fldCavs
        % portID is the port or field the invQ is calculated for
        function [ invQ, portID ] = findCavityMode( obj, cavity )
            leng = find( cavity(2:end) == cavity(1), 1 );
            if isempty(leng)
                leng = length(cavity);
            end
            % first is assumed to be reflection
            M = eye(2,2);
            for iport = 1:leng
                [sinkID, propID] = obj.ifo.props.getPropTo( cavity(iport) );
                if iport < leng
                    outID = cavity(iport+1);
                else
                    outID = cavity(1);
                end
                M = obj.ifo.optics.findABCD( sinkID, outID ) * obj.ifo.props.propList( propID ).propABCD * M;
            end
            invQ = 1/MMTK.EigenMode_ABCD( M );
            portID = cavity(1);
        end
        
        % fillQvals( root, invQ )
        %    set the mode of the field 'root' to invQ, and
        %    set propagate to all fields
        function fillQvals( obj, root, invQ )
            root = obj.ifo.fields.findNamedField( root, -1 );
            obj.ifo.fields.fieldList(root).invQ = invQ;
            % buildTree is used to scan all branches and fill Q's, and tree itself is not necessary
            obj.buildTree( root, @obj.setQvals );
            obj.ifo.optics.fillMissingQ( obj.ifo.props );
            % set status to be -1 if they are not part of the calculation
            obj.markUnusedFields;
        end
        
        function treesIn = findTreesIn( obj, fieldID, all )
            % treesIn = findTreesIn( obj, fieldID, all )
            % returns tree index in which this field is on
            if ~exist( 'all', 'var' )
                all = true;
            end
            treesIn = [];
            for it = 1:length(obj.tree)
                if ~isempty( find( abs(obj.tree{it}) == fieldID, 1 ) )
                    treesIn = [treesIn, it];
                    if ~all
                        return;
                    end
                end
            end
        end
        
        function markUnusedFields( obj )
            % function markUnusedFields( obj )
            % set field.status = -1 if the field is not in any of the ifo tree
            
            for fid = 1 : length( obj.ifo.fields.fieldList )
                if obj.ifo.fields.fieldList(fid).status == 0
                    % check if this is in the tree
                    % tree keeps track of only the beginning of propagators
                    % so if this is the end of a propagator, find the beginning
                    fieldID = obj.ifo.props.getPropFrom( fid );
                    if fieldID <= 0
                        % so this fid is the beginning, so test this fid
                        fieldID = fid;
                    end
                    if isempty( obj.findTreesIn( fieldID, false ) )
                        obj.ifo.fields.fieldList(fid).status = -1;
                    end
                end
            end
        end
        
        function printTrees( obj, all, fid )
            if ~exist( 'all', 'var' )
                all = false;
            end
            if ~exist( 'fid', 'var' )
                fid = 1;
            end
            
            fprintf( fid, '\n=== Information about field propagation tree ===\n\n');
            for it = 1:length(obj.tree)
                if all || obj.tree{it}(end) > 0
                    fprintf(fid, '[%2d]', it);
                    for ib = 1 : length( obj.tree{it} )
                        fprintf(fid, ' %s (%d)', obj.ifo.fields.findName( obj.tree{it}(ib) ), obj.tree{it}(ib) );
                        if ib ~= length( obj.tree{it} )
                            fprintf( fid, '  ->');
                        end
                    end
                    fprintf(fid, '\n');
                end
            end
        end
        
        function printFullTrees( obj, all, fid )
            if ~exist( 'all', 'var' )
                all = false;
            end
            if ~exist( 'fid', 'var' )
                fid = 1;
            end
            
            fprintf( fid, '\n=== Information about field propagation tree : FULL TREE ===\n\n');
            for it = 1:length(obj.fullTree)
                if all || obj.fullTree{it}(end) > 0
                    fprintf(fid, '[%2d]', it);
                    for ib = 1 : length( obj.fullTree{it} )
                        fprintf(fid, ' %s (%d)', obj.ifo.fields.findName( obj.fullTree{it}(ib) ), obj.fullTree{it}(ib) );
                        if ib ~= length( obj.fullTree{it} )
                            fprintf( fid, '  ->');
                        end
                    end
                    fprintf(fid, '\n');
                end
            end
        end
        
        function printComp( obj, fid )
            if ~exist( 'fid', 'var' )
                fid = 1;
            end
            if isempty( obj.compList )
                fprintf(fid, '\n>>>     ==== There is no compound cavities\n');
            else
                fprintf( fid, '\n=== Information about compound cavities ===\n\n');
                for it = 1:length(obj.compList)
                    info = obj.compList(it).printInfo;
                    fprintf(fid, '[%2d]\n%s\n', it, info);
                end
            end
        end
        
        function printClosedCav( obj, fid )
            if ~exist( 'fid', 'var' )
                fid = 1;
            end
            
            if isempty( obj.closedCavs )
                fprintf(fid, '\n>>>     ==== There is no closed cavities\n');
            else
                fprintf(fid, '\n>>>     ==== Closed cavities\n');
                for it = 1:length(obj.closedCavs)
                    fprintf(fid, '[%2d]', it);
                    for ib = 1 : length( obj.closedCavs{it} )
                        fprintf(fid, '  %s (%d)', obj.ifo.fields.findName( obj.closedCavs{it}(ib) ), obj.closedCavs{it}(ib) );
                        if ib ~= length( obj.closedCavs{it} )
                            fprintf(fid, '  ->');
                        end
                    end
                    fprintf(fid, '\n');
                    [qInv, portID] = obj.findCavityMode( obj.closedCavs{it} );
                    fprintf(fid, 'Eigen value at port [%d] : RoC = %g, w = %g, q = %g + i*%g\n', ...
                        portID, 1/real(qInv), MMTK.qinv2W(qInv, obj.ifo.fields.kCR), real(1/qInv), imag(1/qInv) );
                    for iguess = 1 : length( obj.guessList )-1
                        if obj.guessList(iguess).id == it
                            fprintf(fid, 'Round trip loss is %g using guess ID %d\n', ...
                                obj.guessList(iguess).roundtripLoss, iguess );
                            break;
                        end
                    end
                    fprintf(fid, '\n');
                end
            end
        end
        
        function printGuessList( obj, fid )
            if ~exist( 'fid', 'var' )
                fid = 1;
            end
            
            fprintf(fid, '\n>>>     ==== Guess lists\n');
            for ig = 1:length(obj.guessList)
                fprintf(fid, '%s\n', obj.guessList(ig).printInfo);
            end
        end
        
        function printAll( obj, fid )
            if ~exist( 'fid', 'var' )
                fid = 1;
            end
            fileOpened = ischar( fid );
            if fileOpened
                fid = fopen(fid, 'w');
            end
            
            obj.printTrees( true, fid );
            obj.printComp( fid );
            obj.printClosedCav( fid );
            obj.printGuessList( fid );
            
            if fileOpened
                fclose(fid);
            end
        end
        
        function setQvals( obj, srcID )
            % function setQvals( obj, srcID )
            % From srcID, propagate to the sink, then do refl and trans.
            % At each operation, assign q value to the field
            [sinkID, propID] = obj.ifo.props.getPropTo( srcID );
            if (propID > 0) && (sinkID > 0)
                if isnan( obj.ifo.fields.fieldList(sinkID).invQ )
                    % calculate propagation
                    invQ = MMTK.ABCD_qinv( obj.ifo.props.propList(propID).propABCD, obj.ifo.fields.fieldList(srcID).invQ );
                    obj.ifo.fields.fieldList(sinkID).invQ = invQ;
                    
                    [outABCD, outID] = obj.ifo.optics.findReflABCD( sinkID );
                    if outID > 0 && isnan( obj.ifo.fields.fieldList(outID).invQ )
                        obj.ifo.fields.fieldList(outID).invQ = MMTK.ABCD_qinv( outABCD, invQ );
                    end
                    [outABCD, outID] = obj.ifo.optics.findTransABCD( sinkID );
                    if outID > 0 && isnan( obj.ifo.fields.fieldList(outID).invQ )
                        obj.ifo.fields.fieldList(outID).invQ = MMTK.ABCD_qinv( outABCD, invQ );
                    end
                end
            end
        end
        
    end
    
    methods (Static)
        
        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        %
        % Find non overlapping closed loops to identify guess fields
        %
        % closedCavs is a cell array, each element is one closed loop
        % tag, a vector of integers, identifes how many fields are needed to calculate
        % the field outgoing from the node
        %
        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        
        function closedCavs = findClosedCavs( tree )
            
            % find closed loops
            closedCavs = {};
            for ibr = 1 : length( tree )
                branchPath = [ tree{ibr} ];
                branch = branchPath(end);
                % analyze only those branch path which ends on other nodes
                if branch > 0
                    % if this is in other loop, it is used
                    if isempty( find( [closedCavs{:}] == branch, 1 ) )
                        % if the end is somewhere in the current branch path
                        closed = find( branchPath(1:end-1) == branch );
                        if ~isempty( closed )
                            % then this is a closed loop so a cavity
                            numCav = length( closedCavs ) + 1;
                            closedCavs( numCav ) = { branchPath(closed(end):end) };
                        end
                    end
                end
            end
            
        end
        
        % add a new branch path to the tree
        function tree = addBranch( branchPath, tree )
            numBranchs = length( tree ) + 1;
            tree(numBranchs) = { branchPath };
        end
        
        % shrink the branchPath to the last branch point, trBranch(end), and remove this trBranch data
        function [branchPath, trBranch] = back2TrBranch( branchPath, trBranch )
            if isempty( trBranch )
                branchPath = [];
            else
                branchPath = [ branchPath(1:trBranch(end).brIndex), trBranch(end).trNode ];
                trBranch(end) = [];
            end
        end
        
        function dup = duplicate2( longList, shortList )
            for ilocS = 1 : length(shortList)-1
                for ilocL = 1 : length(longList)-1
                    if (longList(ilocL) == shortList(ilocS)) && ...
                            (longList(ilocL+1) == shortList(ilocS+1))
                        dup = true;
                        return;
                    end
                end
            end
            dup = false;
        end
        
        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        %
        % When a cavity beam axis is defined by the beam center on ITM and ETM,
        % the tilt anngles of two mirrors are calculated.
        % The beam center, (xI, yI) and (xE, yE) and tilt angles (thetaI, phiI) and
        % (thetaE, phiE) are all local to the mass.
        %
        % The tilt angles are to be used in addMirror('optTheta', 'optPhi')
        %
        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        
        function [thetaI,phiI,thetaE,phiE] = beam2tilt( xI, yI, xE, yE, invRoCI, invRoCE, L )
            
            dX = xI + xE;
            dY = yI - yE;
            phiE = atan2( yI*invRoCE - dY*(invRoCE-1/L), -xI*invRoCE + dX*(invRoCE-1/L) );
            phiI = atan2( yI*invRoCI - dY/L, xI*invRoCI - dX/L );
            sinPhi = sin( phiE );
            if abs(sinPhi) > 0.5
                thetaE = (yI*invRoCE - dY*(invRoCE-1/L)) / sinPhi;
            else
                thetaE = (-xI*invRoCE + dX*(invRoCE-1/L)) / cos(phiE);
            end
            sinPhi = sin(phiI);
            if abs(sinPhi) > 0.5
                thetaI = (yI*invRoCI - dY/L) / sinPhi;
            else
                thetaI = (xI*invRoCI - dX/L) / cos(phiI);
            end
        end
    end
    
end