% function [merged,x,y, mapLow, mapHigh] = mergeTwoMaps(fileHigh, fileLow, lamMerge, aperture, Nfilter, ITFmag )
%
% merge two phase map data as  
%   merged = lowFreq + high pass filter( highFreq - lowFreq, mergWaveLeng, ITF correction )
%
%  [merged,x,y, mapLow, mapHigh] = mergeTwoMaps('ETM13_S1_self-ref.dat', 'ETM13_CIT.dat', 0.02, 0.32, 2, 1 );
%
% May 15, 2020
%    Updated to allow data set can be empty, just low pass or high pass
%    If the bins are different, large one is used after interpreted
%
% May 17, 2018 : HY updated
%
function [merged,x,y, mapLow, mapHigh] = mergeTwoMaps(fileHigh, fileLow, lamMerge, aperture, Nfilter, ITFmag, mirrorAperture )

eval( UtilTK.setDEFVALS('Nfilter', 2, 'ITFmag', 1) );

if ~exist('mirrorAperture', 'var')
    mirrorAperture = 0;
end

if isempty( fileHigh )
    xh = 0;
else
    [mapHigh, xh, yh] = loadOneDataFile( fileHigh, aperture, 2, mirrorAperture );
end
if isempty( fileLow )
    xl = 0;
else
    [mapLow, xl, yl] = loadOneDataFile( fileLow, aperture, 2, mirrorAperture );
end

% both data are given
if size(xl) > 1
    x = xl;
    y = yl;
    mapHigh = zeros( size(mapLow) );
elseif size(xh) > 1
    x = xh;
    y = yh;
    mapLow = zeros( size(mapHigh) );
elseif length(xl) > 1 && length(xl) > 1
    % if the coordinates of two data sets are different
    if length(xl) ~= length(xh) || xl ~= xh
        if xl(end) > xh(end)
            [xg,yg] = meshgrid( xh, yh );
            mapLow = interp2( xl, yl, mapLow, xg, yg );
            x = xh;
            y = yh;
        else
            [xg,yg] = meshgrid( xl, yl );
            mapHigh = interp2( xh, yh, mapHigh, xg, yg );
            x = xl;
            y = yl;
        end
    end
else
    error( 'Data files are not given\n');
end

merged = mapLow + myHighPass( mapHigh - mapLow, x, y, lamMerge, Nfilter, ITFmag );

if mirrorAperture > 0
    [xg, yg] = meshgrid( x, y );
    merged( xg.^2 + yg.^2 > mirrorAperture^2/4 ) = nan;
end

end

function highPassed = myHighPass( map, x, y, lamMerge, Nfilter, ITFmag )

NaN_Mask = ~isfinite(map);
map(NaN_Mask)=0;

Fcutoff = (x(2)-x(1)) / lamMerge;
filterWgt = max( 1 - UtilTK.lowpassFilter( length(x), length(y), Fcutoff, Nfilter ), 0 );
if ITFmag >= 1 && ITFmag <= 4
    [fx,fy] = meshgrid( ((1:length(x)) - fix(length(x)/2)-1)/length(x), ...
        ((1:length(y)) - fix(length(y)/2)-1)/length(y) );
    filterWgt = filterWgt .* ITFATP( sqrt(fx.^2 + fy.^2) / ((x(2)-x(1))*1000), ITFmag );
end
highPassed = real( fftshift( ifft2 ( ifftshift(filterWgt .* fftshift(fft2(ifftshift(map)))) ) ) );
end
