% function plotEvol( [evol1, ...], {evol1Tag, ...}, [guessIndex1, ...], {guess1Name, ...}, [data1, ...], {dataName1, ...} )
%   Utility function to plot lock/calc output evol.
%
%   evolNTag is the name of evolN
%   guessIndexN is the guess field index, which will corresponds to PRC or SRC or ...
%   dataN is the index in evol{1}, which will be 
%      [1:error, 2:power, 3:delL, 4:phase for delL, 5:kL-gouy, 6:MICH delL]
%      When dataN < 0, semilog is used
%
% August 28, 2018
%   number of guess fields can be arbitrary, not fixed to be 2
%
% May 18, 2018  HY initial addition
%
%{
%      plotEvol( evol, {}, [1,2,3], {'PRC', 'SRC', 'X arm'}, [-1,-2,4,5,6])
%   example : 
      ifo = CCIFOAcc; % or any IFO 
      evol2 = ifo.lock('AccN',2, 'resetG', 1); evol4 = ifo.lock('AccN',4, 'resetG', 1);
      plotEvol({evol2, evol4}, {'AccN=2', 'AccN=4'}, [1,2], {'PRC', 'X arm'}, [-1,-2,4,5,6], {'Error', 'Power', 'Phase', 'kL-gouy', 'MICH'} );
%}

function plotEvol( evols, evolTag, whichGuess, guessName, whichData, dataName )

dataNameTab = {'Error', 'Power', 'delL', 'Phase', 'kL-gouy', 'MICH'};

if ~iscell( evols{1,1} )
    evols = { evols };
end

% name of each evol
if ~exist( 'evolTag', 'var' )
    evolTag = {};
end

% which guess fields
if ~exist( 'whichGuess', 'var' )
    whichGuess = [1,2,3];
end
numGuess = length(whichGuess);
if ~exist( 'guessName', 'var' )
    guessName = cell(1, numGuess);
end

% which data for each guess field
if ~exist('whichData', 'var')
    whichData = [-1,-2,3,5];
end

% check if the data exist
whichDataCheck = sign(whichData) .* min( abs(whichData), size(evols{1}{1},2));
diffData = find( whichData == whichDataCheck );
if length( diffData ) < length( whichDataCheck )
    whichData = whichDataCheck( diffData );
end

if ~exist('dataName', 'var')
    for id = 1 : length(whichData)
        dataName{id} = dataNameTab{ abs(whichData(id)) };
    end
end

% make the x axis (evolution count) to be even
xmax = evols{1}{2}(end);
for id = 2:length(evols)
    xmax = max( xmax, evols{id}{2}(end) );
end
xmax = (floor( xmax/10)+1) * 10;

% subplot( whichData, whichGuess )
numData = length(whichData);
for iplot = 1 : numData * numGuess
    subplot(numData,numGuess,iplot);
    dataIndex = ceil(iplot/numGuess);
    ic = whichGuess(mod(iplot-1,numGuess)+1); ia = whichData(dataIndex);
    if ia < 0
        eval( sprintf( 'semilogy( %s );', plotMulStr( evols, abs(ia), ic, true ) ) );
    else
        eval( sprintf( 'plot( %s );', plotMulStr( evols, ia, ic ) ) );
    end
    axisXY(0, xmax, 1.2);
    grid
    title({guessName{mod(iplot-1,numGuess)+1} ; dataName{dataIndex}})
    if iplot == 1
        if ~isempty( evolTag )
            legend(evolTag);
        end
    end
end
end

function str = plotMulStr( evols, ic, ia, plotAbs )
if exist('plotAbs', 'var') && plotAbs
    absL = 'abs(';
    absR = ')';
else
    absL = '';
    absR = '';
end
str = '';
for id = 1:length( evols )
    str = sprintf( '%sevols{%d}{2}, %sevols{%d}{1}(:,%d,%d)%s', str, id, absL, id, ic, ia, absR);
    switch id
        case 1
            lineMark = ',''x-''';
        case 2
            lineMark = ',''s-''';
        otherwise
            lineMark = ',''+-''';
    end
    str = [str,lineMark];
    if id ~= length( evols )
        str = [str, ','];
    end
end
end

% function axisX( x )
%    x = [xmin, xmax] or [xmin, xmax, ymin, ymax]
function axisXY( xmin, xmax, yscale )
V = axis;
V(1:2) = [xmin, xmax];
if exist('yscale', 'var')
    yvar = (V(4) - V(3))*yscale;
    V(4) = V(3) + yvar;
end
axis(V);
end

