% [fSum, x, y] = zernikeSum( x, y, coef, normRadius, outRadius )
%    argument order changed
%    coef(:,1) = n, coef(:,2) = m, coef(:,3) = weight
%
% calculate sum of zernike terms with normalization radius of normRadius at (x, y) within radius of outRadius
%    fSum = sum( coef( n, m ) * zernike( n, m, x/nr, y/nr ) )
% [x,y] on output are 2 d matrix, and fSum are nan outside of x^2+y^2 > outRadius^2
% coef are used with unnormalized zernike functions.
%
% May 18, 2018 : HY initial registration
%
function [fSum, x, y] = zernikeSum( x, y, coef, normRadius, outRadius )

if min( size(x) ) == 1
    if size(x,2) == 1
        x = x';
    end
end
if min( size(y) ) == 1
    if size(y,2) == 1
        y = y';
    end
end
if size(x,1) == 1 && size(y,1) == 1
    [x,y] = meshgrid( x, y );
elseif size(x) ~= size(y)
    error( 'size x (%d,%d) and size y (%d,%d) are different\n', size(x,1), size(x,2), size(y,1), size(y,2));
end

fSum = zeros( size(x) );
if size( coef, 1 ) > 0
    r = sqrt(x.^2 + y.^2);
    if exist( 'outRadius', 'var' )
        r( r > outRadius ) = nan;
    end
    r = r/normRadius;
    theta = atan2( y, x );
    for inm = 1 : size( coef, 1 )
        if coef(inm) ~= 0
            fSum = fSum + coef(inm, 3) * zernikeFunc( coef(inm,1), coef(inm,2), r, theta, false );
        end
    end
end
    